//**********************************************************************;
//*                                                                    *;
//* INTEL CONFIDENTIAL                                                 *;
//* Copyright 2006-2011 Intel Corporation All Rights Reserved.         *;
//*                                                                    *;
//* The source code contained or described herein and all documents    *;
//* related to the source code ("Material") are owned by Intel         *;
//* Corporation or its suppliers or licensors. Title to the Material   *;
//* remains with Intel Corporation or its suppliers and licensors.     *;
//* The Material contains trade secrets and proprietary and            *;
//* confidential information of Intel or its suppliers and licensors.  *;
//* The Material is protected by worldwide copyright and trade secret  *;
//* laws and treaty provisions. No part of the Material may be used,   *;
//* copied, reproduced, modified, published, uploaded, posted,         *;
//* transmitted, distributed, or disclosed in any way without          *;
//* Intels prior express written permission.                          *;
//*                                                                    *;
//* No license under any patent, copyright, trade secret or other      *;
//* intellectual property right is granted to or conferred upon you    *;
//* by disclosure or delivery of the Materials, either expressly,      *;
//* by implication, inducement, estoppel or otherwise. Any license     *;
//* under such intellectual property rights must be express and        *;
//* approved by Intel in writing.                                      *;
//*                                                                    *;
//**********************************************************************;
 
#ifndef TPM20_H
#define TPM20_H 

#include <stdio.h>
#include "TPM_Types.h"
#pragma pack (1)
#define CPP_EXTERN
#if 0
#define MAX_SESSIONS 3    

// Input structure for session data for a single session,
typedef struct {
    TPMI_SH_AUTH_SESSION sessionHandle;
    TPM2B_NONCE *nonce;
    TPMA_SESSION sessionAttributes;
    TPM2B_AUTH *hmac;
} TPM_CMD_SESSION_DATA_IN ;

// Input structure for sessions data.
typedef struct {
    UINT8 numSessions;
    TPM_CMD_SESSION_DATA_IN sessions[MAX_SESSIONS];
} TPM_CMD_SESSIONS_DATA_IN;

// Output structure for session data for a single session.
typedef struct {
    TPM2B_NONCE *nonceTpm;
    TPMA_SESSION sessionAttributes;
    TPM2B_AUTH  *hmac;
} TPM_CMD_SESSION_DATA_OUT;

// Output structure for sessions data.
typedef struct {
    UINT8 numSessions;
    TPM_CMD_SESSION_DATA_OUT sessions[MAX_SESSIONS];
} TPM_CMD_SESSIONS_DATA_OUT;
#endif

//
// Generic header
// 
typedef struct _TPM20_Header_In {
  TPM_ST tag;
  UINT32 commandSize;
  UINT32 commandCode;
  UINT8 otherData;
} TPM20_Header_In;
                              
typedef struct _TPM20_Header_Out {
  TPM_ST tag;
  UINT32 responseSize;
  UINT32 responseCode;
  UINT8 otherData;
} TPM20_Header_Out;


//
// TPM20 startup input/output structures
//
typedef struct {
  TPM_ST tag;
  UINT32 commandSize;
  TPM_CC commandCode;
  UINT16 startupType;
} TPM20_Startup_In;

typedef struct {
  TPM_ST tag;
  UINT32 responseSize;
  UINT32 responseCode; 
} TPM20_Startup_Out;

//
// TPM20 Shutdown input/output structures
//
typedef struct {
  TPM_ST tag;
  UINT32 commandSize;
  TPM_CC commandCode;
  UINT8 otherData;
//
// NOTE: other input data consists of the following fields.  But it can't be
// specified here because some structures can vary in size.
//
//  TPM_CMD_SESSION_DATA_IN[];
//
// TPM_SU shutdownType;
//
} TPM20_Shutdown_In;

typedef struct {
  TPM_ST tag;
  UINT32 responseSize;
  UINT32 responseCode;
  UINT8 otherData;
//
// NOTE: other output data consists of the following fields.  But it can't be
// specified here because some structures can vary in size or may not
// even be present.
//
// TPM_CMD_SESSION_DATA_OUT[] sessions;
//
} TPM20_Shutdown_Out;

//
// TPM20 Selftest input/output structures
//
typedef struct  {
  TPM_ST tag;
  UINT32 commandSize;
  TPM_CC commandCode;
  UINT8 fullTest;
  UINT8 otherData;
//
// NOTE: other input data consists of the following fields.  But it can't be
// specified here because some structures can vary in size.
//
//  TPM_CMD_SESSION_DATA_IN[];
//
} TPM20_Selftest_In;

typedef struct {
  TPM_ST tag;
  UINT32 responseSize;
  UINT32 responseCode; 
  UINT8 otherData;
//
// NOTE: other output data consists of the following fields.  But it can't be
// specified here because some structures can vary in size or may not
// even be present.
//
// TPM_CMD_SESSION_DATA_OUT[] sessions;
//
} TPM20_Selftest_Out;


//
// TPM20 IncrementalSelfTest input/output structures
//
typedef struct  {
  TPM_ST tag;
  UINT32 commandSize;
  TPM_CC commandCode;
  TPML_ALG toTest;
  UINT8 otherData;
//
// NOTE: other input data consists of the following fields.  But it can't be
// specified here because some structures can vary in size.
//
//  TPM_CMD_SESSION_DATA_IN[];
//
} TPM20_IncrementalSelfTest_In;

typedef struct {
  TPM_ST tag;
  UINT32 responseSize;
  UINT32 responseCode; 
  UINT8 otherData;
//
// NOTE: other output data consists of the following fields.  But it can't be
// specified here because some structures can vary in size or may not
// even be present.
//
// TPML_ALG toDoList;
//
// TPM_CMD_SESSION_DATA_OUT[] sessions;
//
} TPM20_IncrementalSelfTest_Out;

//
// TPM20 GetTestResult input/output structures
//
typedef struct  {
  TPM_ST tag;
  UINT32 commandSize;
  TPM_CC commandCode;
  UINT8 otherData;
//
// NOTE: other input data consists of the following fields.  But it can't be
// specified here because some structures can vary in size.
//
//  TPM_CMD_SESSION_DATA_IN[];
//
} TPM20_GetTestResult_In;

typedef struct {
  TPM_ST tag;
  UINT32 responseSize;
  UINT32 responseCode; 
  UINT8 otherData;
//
// NOTE: other output data consists of the following fields.  But it can't be
// specified here because some structures can vary in size or may not
// even be present.
//
// TPM2B_MAX_BUFFER outData;
// TPM_RC testResult;
//
// TPM_CMD_SESSION_DATA_OUT[] sessions;
//
} TPM20_GetTestResult_Out;

//
// TPM20 StartAuthSession input/output structures
//
typedef struct {
  TPM_ST tag;
  UINT32 commandSize;
  TPM_CC commandCode;
  TPMI_DH_OBJECT tpmKey;
  TPMI_DH_ENTITY bind;
  UINT8 otherData;
//
// NOTE: other input data consists of the following fields.  But it can't be
// specified here because some structures can vary in size.
//
//  TPM_CMD_SESSION_DATA_IN[];
//
} TPM20_StartAuthSession_In;

typedef struct {
  TPM_ST tag;
  UINT32 responseSize;
  UINT32 responseCode;
  UINT8 otherData;
//
// NOTE: other output data consists of the following fields.  But it can't be
// specified here because some structures can vary in size or may not
// even be present.
//
// TPMI_SH_AUTH_SESSION sessionHandle;
// TPM2B_NONCE nonceTpm;
//
// TPM_CMD_SESSION_DATA_OUT[] sessions;
//
} TPM20_StartAuthSession_Out;


//
// TPM20 PolicyRestart input/output structures
//
typedef struct {
  TPM_ST tag;
  UINT32 commandSize;
  TPM_CC commandCode;
  TPMI_SH_POLICY sessionHandle;
  UINT8 otherData;
//
// NOTE: other input data consists of the following fields.  But it can't be
// specified here because some structures can vary in size.
//
//  TPM_CMD_SESSION_DATA_IN[];
//
} TPM20_PolicyRestart_In;

typedef struct {
  TPM_ST tag;
  UINT32 responseSize;
  UINT32 responseCode;
  UINT8 otherData;
//
// NOTE: other output data consists of the following fields.  But it can't be
// specified here because some structures can vary in size or may not
// even be present.
//
// TPM_CMD_SESSION_DATA_OUT[] sessions;
//
} TPM20_PolicyRestart_Out;

//
// TPM20 Create input/output structures
//
typedef struct {
  TPM_ST tag;
  UINT32 commandSize;
  TPM_CC commandCode;
  TPMI_DH_OBJECT parentHandle;
  UINT8 otherData;
//


// NOTE: other input data consists of the following fields.  But it can't be
// specified here because some structures can vary in size.
//
//  TPM_CMD_SESSION_DATA_IN[];
//
//  TPM2B_SENSITIVE_CREATE inSenstive;
//  TPM2B_PUBLIC inPublic;
//  TPM2B_DATA outsideInfo;
//  TPML_PCR_SELECTION creationPCR;
//
} TPM20_Create_In;

typedef struct {
  TPM_ST tag;
  UINT32 responseSize;
  UINT32 responseCode;
  UINT8 otherData;
//
// NOTE: other output data consists of the following fields.  But it can't be
// specified here because some structures can vary in size or may not
// even be present.
//
// TPM2B_PRIVATE outPrivate;
// TPM2B_PUBLIC outPublic;
// TPM2B_CREATION_DATA creationData;
// TPM2B_DIGEST creationHash;
// TPMT_TK_CREATION creationTicket;
//
// TPM_CMD_SESSION_DATA_OUT[] sessions;
//
} TPM20_Create_Out;


//
// TPM20 Load input/output structures
//
typedef struct {
  TPM_ST tag;
  UINT32 commandSize;
  TPM_CC commandCode;
  TPMI_DH_OBJECT parentHandle;
  UINT8 otherData;
//
// NOTE: other input data consists of the following fields.  But it can't be
// specified here because some structures can vary in size.
//
//  TPM_CMD_SESSION_DATA_IN[];
//
//  TPM2B_PRIVATE inPrivate;
//  TPM2B_PUBLIC inPublic;
//
  
} TPM20_Load_In;

typedef struct {
  TPM_ST tag;
  UINT32 responseSize;
  UINT32 responseCode;
  UINT8 otherData;
//
// NOTE: other output data consists of the following fields.  But it can't be
// specified here because some structures can vary in size or may not
// even be present.
//
// TPM_HANDLE objectHandle;
//
// TPM2B_NAME name;
//
// TPM_CMD_SESSION_DATA_OUT[] sessions;
//
} TPM20_Load_Out;


//
// TPM20 LoadExternal input/output structures
//
typedef struct {
  TPM_ST tag;
  UINT32 commandSize;
  TPM_CC commandCode;
  UINT8 otherData;
//
// NOTE: other input data consists of the following fields.  But it can't be
// specified here because some structures can vary in size.
//
//  TPM_CMD_SESSION_DATA_IN[];
//
//  TPM2B_SENSITIVE inPrivate;
//  TPM2B_PUBLIC inPublic;
//  TPMI_RH_HIERARCHY hierarchy;
//
  
} TPM20_LoadExternal_In;

typedef struct {
  TPM_ST tag;
  UINT32 responseSize;
  UINT32 responseCode;
  UINT8 otherData;
//
// NOTE: other output data consists of the following fields.  But it can't be
// specified here because some structures can vary in size or may not
// even be present.
//
// TPM2B_NAME name;
//
// TPM_CMD_SESSION_DATA_OUT[] sessions;
//
} TPM20_LoadExternal_Out;


//
// TPM20 ReadPublic input/output structures
//
typedef struct  {
  TPM_ST tag;
  UINT32 commandSize;
  TPM_CC commandCode;
  TPMI_DH_OBJECT objectHandle;
  UINT8 otherData;
//
// NOTE: other input data consists of the following fields.  But it can't be
// specified here because some structures can vary in size.
//
//  TPM_CMD_SESSION_DATA_IN[];
//
} TPM20_ReadPublic_In;

typedef struct {
  TPM_ST tag;
  UINT32 responseSize;
  UINT32 responseCode; 
  UINT8 otherData;
//
// NOTE: other output data consists of the following fields.  But it can't be
// specified here because some structures can vary in size or may not
// even be present.
//
// TPM2B_PUBLIC outPublic;
// TPM2B_NAME name;
// TPM2B_NAME qualifiedName;
//
// TPM_CMD_SESSION_DATA_OUT[] sessions;
//
} TPM20_ReadPublic_Out;


//
// TPM20 ActivateCredential input/output structures
//
typedef struct  {
  TPM_ST tag;
  UINT32 commandSize;
  TPM_CC commandCode;
  TPMI_DH_OBJECT activateHandle;
  TPMI_DH_OBJECT keyHandle;
  UINT8 otherData;
//
// NOTE: other input data consists of the following fields.  But it can't be
// specified here because some structures can vary in size.
//
//  TPM_CMD_SESSION_DATA_IN[];
//
// TPM2B_ID_OBJECT credentialBlob;
// TPM2B_ENCRYPTED_SECRET secret;
//
} TPM20_ActivateCredential_In;

typedef struct {
  TPM_ST tag;
  UINT32 responseSize;
  UINT32 responseCode; 
  UINT8 otherData;
//
// NOTE: other output data consists of the following fields.  But it can't be
// specified here because some structures can vary in size or may not
// even be present.
//
// TPM2B_DIGEST certInfo;
//
// TPM_CMD_SESSION_DATA_OUT[] sessions;
//
} TPM20_ActivateCredential_Out;


//
// TPM20 MakeCredential input/output structures
//
typedef struct  {
  TPM_ST tag;
  UINT32 commandSize;
  TPM_CC commandCode;
  TPMI_DH_OBJECT handle;
  UINT8 otherData;
//
// NOTE: other input data consists of the following fields.  But it can't be
// specified here because some structures can vary in size.
//
//  TPM_CMD_SESSION_DATA_IN[];
//
// TPM2B_DIGEST credential;
// TPM2B_NAME objectName;
//
} TPM20_MakeCredential_In;

typedef struct {
  TPM_ST tag;
  UINT32 responseSize;
  UINT32 responseCode; 
  UINT8 otherData;
//
// NOTE: other output data consists of the following fields.  But it can't be
// specified here because some structures can vary in size or may not
// even be present.
//
// TPM2B_ID_OBJECT credentialBlob;
// TPM2B_ENCRYPTED_SECRET secret;
//
// TPM_CMD_SESSION_DATA_OUT[] sessions;
//
} TPM20_MakeCredential_Out;


//
// TPM20 Unseal input/output structures
//
typedef struct  {
  TPM_ST tag;
  UINT32 commandSize;
  TPM_CC commandCode;
  TPMI_DH_OBJECT itemHandle;
  UINT8 otherData;
//
// NOTE: other input data consists of the following fields.  But it can't be
// specified here because some structures can vary in size.
//
//  TPM_CMD_SESSION_DATA_IN[];
//
} TPM20_Unseal_In;

typedef struct {
  TPM_ST tag;
  UINT32 responseSize;
  UINT32 responseCode;
  UINT8 otherData;
//
// NOTE: other output data consists of the following fields.  But it can't be
// specified here because some structures can vary in size or may not
// even be present.
//
// TPM2B_SENSITIVE_DATA outData;
//
// TPM_CMD_SESSION_DATA_OUT[] sessions;
//
} TPM20_Unseal_Out;


//
// TPM20 ObjectChangeAuth input/output structures
//
typedef struct  {
  TPM_ST tag;
  UINT32 commandSize;
  TPM_CC commandCode;
  TPMI_DH_OBJECT objectHandle;
  TPMI_DH_OBJECT parentHandle;
  UINT8 otherData;
//
// NOTE: other input data consists of the following fields.  But it can't be
// specified here because some structures can vary in size.
//
// TPM_CMD_SESSION_DATA_IN[];
//
// TPM2B_AUTH newAuth;
//
} TPM20_ObjectChangeAuth_In;

typedef struct {
  TPM_ST tag;
  UINT32 responseSize;
  UINT32 responseCode; 
  UINT8 otherData;
//
// NOTE: other output data consists of the following fields.  But it can't be
// specified here because some structures can vary in size or may not
// even be present.
//
// TPM2B_PRIVATE outPrivate;
//
// TPM_CMD_SESSION_DATA_OUT[] sessions;
//
} TPM20_ObjectChangeAuth_Out;


//
// TPM20 Duplicate input/output structures
//
typedef struct  {
  TPM_ST tag;
  UINT32 commandSize;
  TPM_CC commandCode;
  TPMI_DH_OBJECT objectHandle;
  TPMI_DH_OBJECT newParentHandle;
  UINT8 otherData;
//
// NOTE: other input data consists of the following fields.  But it can't be
// specified here because some structures can vary in size.
//
// TPM_CMD_SESSION_DATA_IN[];
//
// TPM2B_DATA encryptionKeyIn;
// TPMT_SYM_DEF_OBJECT symmetricAlg;
//
} TPM20_Duplicate_In;

typedef struct {
  TPM_ST tag;
  UINT32 responseSize;
  UINT32 responseCode; 
  UINT8 otherData;
//
// NOTE: other output data consists of the following fields.  But it can't be
// specified here because some structures can vary in size or may not
// even be present.
//
// TPM2B_DATA encryptionKeyOut;
// TPM2B_PRIVATE duplicate;
// TPM2B_ENCRYPTED_SECRET outSymSeed;
//
// TPM_CMD_SESSION_DATA_OUT[] sessions;
//
} TPM20_Duplicate_Out;


//
// TPM20 Rewrap input/output structures
//
typedef struct  {
  TPM_ST tag;
  UINT32 commandSize;
  TPM_CC commandCode;
  TPMI_DH_OBJECT oldParent;
  TPMI_DH_OBJECT newParent;
  UINT8 otherData;
//
// NOTE: other input data consists of the following fields.  But it can't be
// specified here because some structures can vary in size.
//
// TPM_CMD_SESSION_DATA_IN[];
//
// TPM2B_PRIVATE inDuplicate;
// TPM2B_ENCRYPTED_SECRET inSymSeed;
//
} TPM20_Rewrap_In;

typedef struct {
  TPM_ST tag;
  UINT32 responseSize;
  UINT32 responseCode; 
  UINT8 otherData;
//
// NOTE: other output data consists of the following fields.  But it can't be
// specified here because some structures can vary in size or may not
// even be present.
//
// TPM2B_PRIVATE outDuplicate;
// TPM2B_ENCRYPTED_SECRET outSymSeed;
//
// TPM_CMD_SESSION_DATA_OUT[] sessions;
//
} TPM20_Rewrap_Out;


//
// TPM20 Import input/output structures
//
typedef struct  {
  TPM_ST tag;
  UINT32 commandSize;
  TPM_CC commandCode;
  TPMI_DH_OBJECT parentHandle;
  UINT8 otherData;
//
// NOTE: other input data consists of the following fields.  But it can't be
// specified here because some structures can vary in size.
//
// TPM_CMD_SESSION_DATA_IN[];
//
// TPM2B_DATA encryptionKey;
// TPM2B_PUBLIC objectPublic;
// TPM2B_PRIVATE duplicate;
// TPM2B_ENCRYPTED_SECRET inSymSeed;
// TPMT_SYM_DEF_OBJECT symmetricAlg;
//
} TPM20_Import_In;

typedef struct {
  TPM_ST tag;
  UINT32 responseSize;
  UINT32 responseCode; 
  UINT8 otherData;
//
// NOTE: other output data consists of the following fields.  But it can't be
// specified here because some structures can vary in size or may not
// even be present.
//
// TPM2B_PRIVATE outPrivate;
//
// TPM_CMD_SESSION_DATA_OUT[] sessions;
//
} TPM20_Import_Out;


//
// TPM20 RSA_Encrypt input/output structures
//
typedef struct  {
  TPM_ST tag;
  UINT32 commandSize;
  TPM_CC commandCode;
  TPMI_DH_OBJECT keyHandle;
  UINT8 otherData;
//
// NOTE: other input data consists of the following fields.  But it can't be
// specified here because some structures can vary in size.
//
// TPM_CMD_SESSION_DATA_IN[];
//
// TPM2B_PUBLIC_KEY_RSA message;
// TPMT_RSA_DECRYPT inScheme;
// TPM2B_DATA label;
//
} TPM20_RSA_Encrypt_In;

typedef struct {
  TPM_ST tag;
  UINT32 responseSize;
  UINT32 responseCode; 
  UINT8 otherData;
//
// NOTE: other output data consists of the following fields.  But it can't be
// specified here because some structures can vary in size or may not
// even be present.
//
// TPM2B_PUBLIC_KEY_RSA outData;
//
// TPM_CMD_SESSION_DATA_OUT[] sessions;
//
} TPM20_RSA_Encrypt_Out;


//
// TPM20 RSA_Decrypt input/output structures
//
typedef struct  {
  TPM_ST tag;
  UINT32 commandSize;
  TPM_CC commandCode;
  TPMI_DH_OBJECT keyHandle;
  UINT8 otherData;
//
// NOTE: other input data consists of the following fields.  But it can't be
// specified here because some structures can vary in size.
//
// TPM_CMD_SESSION_DATA_IN[];
//
// TPM2B_PUBLIC_KEY_RSA cipherText;
// TPMT_RSA_DECRYPT inScheme;
// TPM2B_DATA label;
//
} TPM20_RSA_Decrypt_In;

typedef struct {
  TPM_ST tag;
  UINT32 responseSize;
  UINT32 responseCode; 
  UINT8 otherData;
//
// NOTE: other output data consists of the following fields.  But it can't be
// specified here because some structures can vary in size or may not
// even be present.
//
// TPM2B_PUBLIC_KEY_RSA message;
//
// TPM_CMD_SESSION_DATA_OUT[] sessions;
//
} TPM20_RSA_Decrypt_Out;


//
// TPM20 ECDH_KeyGen input/output structures
//
typedef struct  {
  TPM_ST tag;
  UINT32 commandSize;
  TPM_CC commandCode;
  TPMI_DH_OBJECT keyHandle;
  UINT8 otherData;
//
// NOTE: other input data consists of the following fields.  But it can't be
// specified here because some structures can vary in size.
//
// TPM_CMD_SESSION_DATA_IN[];
//
} TPM20_ECDH_KeyGen_In;

typedef struct {
  TPM_ST tag;
  UINT32 responseSize;
  UINT32 responseCode; 
  UINT8 otherData;
//
// NOTE: other output data consists of the following fields.  But it can't be
// specified here because some structures can vary in size or may not
// even be present.
//
// TPM2B_ECC_POINT zPoint;
// TPM2B_ECC_POINT pubPoint;
//
// TPM_CMD_SESSION_DATA_OUT[] sessions;
//
} TPM20_ECDH_KeyGen_Out;


//
// TPM20 ECDH_ZGen input/output structures
//
typedef struct  {
  TPM_ST tag;
  UINT32 commandSize;
  TPM_CC commandCode;
  TPMI_DH_OBJECT keyHandle;
  UINT8 otherData;
//
// NOTE: other input data consists of the following fields.  But it can't be
// specified here because some structures can vary in size.
//
// TPM_CMD_SESSION_DATA_IN[];
//
// TPM2B_ECC_POINT inPoint;
//
} TPM20_ECDH_ZGen_In;

typedef struct {
  TPM_ST tag;
  UINT32 responseSize;
  UINT32 responseCode; 
  UINT8 otherData;
//
// NOTE: other output data consists of the following fields.  But it can't be
// specified here because some structures can vary in size or may not
// even be present.
//
// TPM2B_ECC_POINT outPoint;
//
// TPM_CMD_SESSION_DATA_OUT[] sessions;
//
} TPM20_ECDH_ZGen_Out;


//
// TPM20 ECC_Parameters input/output structures
//
typedef struct  {
  TPM_ST tag;
  UINT32 commandSize;
  TPM_CC commandCode;
  UINT8 otherData;
//
// NOTE: other input data consists of the following fields.  But it can't be
// specified here because some structures can vary in size.
//
// TPMI_ECC_CURVE curveID;
//
// TPM_CMD_SESSION_DATA_IN[];
//
} TPM20_ECC_Parameters_In;

typedef struct {
  TPM_ST tag;
  UINT32 responseSize;
  UINT32 responseCode; 
  UINT8 otherData;
//
// NOTE: other output data consists of the following fields.  But it can't be
// specified here because some structures can vary in size or may not
// even be present.
//
// TPMS_ALGORITHM_DETAIL_ECC parameters;
//
// TPM_CMD_SESSION_DATA_OUT[] sessions;
//
} TPM20_ECC_Parameters_Out;


//
// TPM20 SM2_ZGen input/output structures
//
typedef struct  {
  TPM_ST tag;
  UINT32 commandSize;
  TPM_CC commandCode;
  TPMI_DH_OBJECT keyA;
  UINT8 otherData;
//
// NOTE: other input data consists of the following fields.  But it can't be
// specified here because some structures can vary in size.
//
// TPM_CMD_SESSION_DATA_IN[];
//
// TPM2B_ECC_POINT inQsB;
// TPM2B_ECC_POINT inQeB;
// UINT16 counter;
//
} TPM20_SM2_ZGen_In;

typedef struct {
  TPM_ST tag;
  UINT32 responseSize;
  UINT32 responseCode; 
  UINT8 otherData;
//
// NOTE: other output data consists of the following fields.  But it can't be
// specified here because some structures can vary in size or may not
// even be present.
//
// TPM2B_ECC_POINT outZ;
//
// TPM_CMD_SESSION_DATA_OUT[] sessions;
//
} TPM20_SM2_ZGen_Out;


//
// TPM20 EncryptDecrypt input/output structures
//
typedef struct  {
  TPM_ST tag;
  UINT32 commandSize;
  TPM_CC commandCode;
  TPMI_DH_OBJECT keyHandle;
  UINT8 otherData;
//
// NOTE: other input data consists of the following fields.  But it can't be
// specified here because some structures can vary in size.
//
// TPM_CMD_SESSION_DATA_IN[];
//
// TPMI_YES_NO decrypt;
// TPMI_ALG_SYM_MODE mode;
// TPM2B_IV IvIn;
// TPM2B_MAX_BUFFER inData;
//
} TPM20_EncryptDecrypt_In;

typedef struct {
  TPM_ST tag;
  UINT32 responseSize;
  UINT32 responseCode; 
  UINT8 otherData;
//
// NOTE: other output data consists of the following fields.  But it can't be
// specified here because some structures can vary in size or may not
// even be present.
//
// TPM2B_MAX_BUFFER outData;
// TPM2B_IV IvOut;
//
// TPM_CMD_SESSION_DATA_OUT[] sessions;
//
} TPM20_EncryptDecrypt_Out;


//
// TPM20 Hash input/output structures
//
typedef struct  {
  TPM_ST tag;
  UINT32 commandSize;
  TPM_CC commandCode;
  UINT8 otherData;
//
// NOTE: other input data consists of the following fields.  But it can't be
// specified here because some structures can vary in size.
//
// TPM_CMD_SESSION_DATA_IN[];
//
// TPM2B_MAX_BUFFER data;
// TPMI_ALG_HASH hashAlg;
// TPMI_RH_hIERARCHY hierarchy;
//
} TPM20_Hash_In;

typedef struct {
  TPM_ST tag;
  UINT32 responseSize;
  UINT32 responseCode; 
  UINT8 otherData;
//
// NOTE: other output data consists of the following fields.  But it can't be
// specified here because some structures can vary in size or may not
// even be present.
//
// TPM2B_DIGEST outHash;
// TPMT_TK_HASHCHECK validation;
//
// TPM_CMD_SESSION_DATA_OUT[] sessions;
//
} TPM20_Hash_Out;


//
// TPM20 HMAC input/output structures
//
typedef struct  {
  TPM_ST tag;
  UINT32 commandSize;
  TPM_CC commandCode;
  TPMI_DH_OBJECT handle;
  UINT8 otherData;
//
// NOTE: other input data consists of the following fields.  But it can't be
// specified here because some structures can vary in size.
//
// TPM_CMD_SESSION_DATA_IN[];
//
// TPM2B_MAX_BUFFER buffer;
// TPMI_ALG_HASH hashAlg;
//
} TPM20_HMAC_In;

typedef struct {
  TPM_ST tag;
  UINT32 responseSize;
  UINT32 responseCode; 
  UINT8 otherData;
//
// NOTE: other output data consists of the following fields.  But it can't be
// specified here because some structures can vary in size or may not
// even be present.
//
// TPM2B_DIGEST outHMAC;
//
// TPM_CMD_SESSION_DATA_OUT[] sessions;
//
} TPM20_HMAC_Out;


//
// TPM20 GetRandom input/output structures
//
typedef struct {
  TPM_ST tag;
  UINT32 commandSize;
  TPM_CC commandCode;
  UINT8 otherData;
//
// NOTE: other input data consists of the following fields.  But it can't be
// specified here because some structures can vary in size.
//
//  TPM_CMD_SESSION_DATA_IN[];
//
//  UINT16 bytesRequested;
//
} TPM20_GetRandom_In;

typedef struct {
  TPM_ST tag;
  UINT32 responseSize;
  UINT32 responseCode;
  UINT8 otherData;
//
// NOTE: other output data consists of the following fields.  But it can't be
// specified here because some structures can vary in size or may not
// even be present.
//
//  TPM2B_DIGEST randomBytes;
//
} TPM20_GetRandom_Out;


//
// TPM20 StirRandom input/output structures
//
typedef struct  {
  TPM_ST tag;
  UINT32 commandSize;
  TPM_CC commandCode;
  UINT8 otherData;
//
// NOTE: other input data consists of the following fields.  But it can't be
// specified here because some structures can vary in size.
//
// TPM_CMD_SESSION_DATA_IN[];
//
// TPM2B_SENSITIVE_DATA inData;
//
} TPM20_StirRandom_In;

typedef struct {
  TPM_ST tag;
  UINT32 responseSize;
  UINT32 responseCode; 
  UINT8 otherData;
//
// NOTE: other output data consists of the following fields.  But it can't be
// specified here because some structures can vary in size or may not
// even be present.
//
// TPM_CMD_SESSION_DATA_OUT[] sessions;
//
} TPM20_StirRandom_Out;


//
// TPM20 HMAC_Start input/output structures
//
typedef struct  {
  TPM_ST tag;
  UINT32 commandSize;
  TPM_CC commandCode;
  TPMI_DH_OBJECT handle;
  UINT8 otherData;
//
// NOTE: other input data consists of the following fields.  But it can't be
// specified here because some structures can vary in size.
//
// TPM_CMD_SESSION_DATA_IN[];
//
// TPM2B_AUTH auth;
// TPMI_ALG_HASH hashAlg;
//
} TPM20_HMAC_Start_In;

typedef struct {
  TPM_ST tag;
  UINT32 responseSize;
  UINT32 responseCode; 
  UINT8 otherData;
//
// NOTE: other output data consists of the following fields.  But it can't be
// specified here because some structures can vary in size or may not
// even be present.
//
// TPMI_DH_OBJECT sequenceHandle;
//
// TPM_CMD_SESSION_DATA_OUT[] sessions;
//
} TPM20_HMAC_Start_Out;


//
// TPM20 HashSequenceStart input/output structures
//
typedef struct {
  TPM_ST tag;
  UINT32 commandSize;
  TPM_CC commandCode;
  UINT8 otherData;
//
// NOTE: other input data consists of the following fields.  But it can't be
// specified here because some structures can vary in size.
//
//  TPM_CMD_SESSION_DATA_IN[];
//
//  TPM2B_AUTH auth;
//  TPMI_ALG_HASH hashAlg;
//
} TPM20_HashSequenceStart_In;

typedef struct {
  TPM_ST tag;
  UINT32 responseSize;
  UINT32 responseCode;
  UINT8 otherData;
//
// NOTE: other output data consists of the following fields.  But it can't be
// specified here because some structures can vary in size or may not
// even be present.
//
//  TPMI_DH_OBJECT sequenceHandle;
//
// TPM_CMD_SESSION_DATA_OUT[] sessions;
//
} TPM20_HashSequenceStart_Out;


//
// TPM20 SequenceUpdate input/output structures
//
typedef struct {
  TPM_ST tag;
  UINT32 commandSize;
  TPM_CC commandCode;
  TPMI_DH_OBJECT sequenceHandle;
  UINT8 otherData;
//
// NOTE: other input data consists of the following fields.  But it can't be
// specified here because some structures can vary in size.
//
//  TPM_CMD_SESSION_DATA_IN[];
//
//  TPM2B_MAX_BUFFER buffer;
//
} TPM20_SequenceUpdate_In;

typedef struct {
  TPM_ST tag;
  UINT32 responseSize;
  UINT32 responseCode;
  UINT8 otherData;
//
// NOTE: other output data consists of the following fields.  But it can't be
// specified here because some structures can vary in size or may not
// even be present.
//
// TPM_CMD_SESSION_DATA_OUT[] sessions;
//
} TPM20_SequenceUpdate_Out;


//
// TPM20 SequenceComplete input/output structures
//
typedef struct {
  TPM_ST tag;
  UINT32 commandSize;
  TPM_CC commandCode;
  TPMI_DH_OBJECT sequenceHandle;
  UINT8 otherData;
//
// NOTE: other input data consists of the following fields.  But it can't be
// specified here because some structures can vary in size.
//
//  TPM_CMD_SESSION_DATA_IN[];
//
//  TPM2B_MAX_BUFFER buffer;
//  TPMI_RH_HIERARCHY hierarchy;
//
} TPM20_SequenceComplete_In;

typedef struct {
  TPM_ST tag;
  UINT32 responseSize;
  UINT32 responseCode;
  UINT8 otherData;
//
// NOTE: other output data consists of the following fields.  But it can't be
// specified here because some structures can vary in size or may not
// even be present.
//
//  TPM2B_DIGEST result;
//  TPMT_TK_HASHCHECK validation;
//
// TPM_CMD_SESSION_DATA_OUT[] sessions;
//
} TPM20_SequenceComplete_Out;


//
// TPM20 EventSequenceComplete input/output structures
//
typedef struct {
  TPM_ST tag;
  UINT32 commandSize;
  TPM_CC commandCode;
  TPMI_DH_PCR pcrHandle;
  TPMI_DH_OBJECT sequenceHandle;
  UINT8 otherData;
//
// NOTE: other input data consists of the following fields.  But it can't be
// specified here because some structures can vary in size.
//
//  TPM_CMD_SESSION_DATA_IN[];
//
//  TPM2B_MAX_BUFFER buffer;
//
} TPM20_EventSequenceComplete_In;

typedef struct {
  TPM_ST tag;
  UINT32 responseSize;
  UINT32 responseCode;
  UINT8 otherData;
//
// NOTE: other output data consists of the following fields.  But it can't be
// specified here because some structures can vary in size or may not
// even be present.
//
//  TPM2B_DIGEST results;
//
// TPM_CMD_SESSION_DATA_OUT[] sessions;
//
} TPM20_EventSequenceComplete_Out;


//
// TPM20 Certify input/output structures
//
typedef struct  {
  TPM_ST tag;
  UINT32 commandSize;
  TPM_CC commandCode;
  TPMI_DH_OBJECT objectHandle;
  TPMI_DH_OBJECT signHandle;
  UINT8 otherData;
//
// NOTE: other input data consists of the following fields.  But it can't be
// specified here because some structures can vary in size.
//
// TPM_CMD_SESSION_DATA_IN[];
//
// TPM2B_DATA qualifyingData;
// TPMT_SIG_SCHEME inScheme;
//
} TPM20_Certify_In;

typedef struct {
  TPM_ST tag;
  UINT32 responseSize;
  UINT32 responseCode; 
  UINT8 otherData;
//
// NOTE: other output data consists of the following fields.  But it can't be
// specified here because some structures can vary in size or may not
// even be present.
//
// TPM2B_ATTEST certifyInfo;
// TPMT_SIGNATURE signature;
//
// TPM_CMD_SESSION_DATA_OUT[] sessions;
//
} TPM20_Certify_Out;


//
// TPM20 CertifyCreation input/output structures
//
typedef struct  {
  TPM_ST tag;
  UINT32 commandSize;
  TPM_CC commandCode;
  TPMI_DH_OBJECT signHandle;
  TPMI_DH_OBJECT objectHandle;
  UINT8 otherData;
//
// NOTE: other input data consists of the following fields.  But it can't be
// specified here because some structures can vary in size.
//
// TPM_CMD_SESSION_DATA_IN[];
//
// TPM2B_DATA qualifyingData;
// TPM2B_DIGEST creationHash;
// TPMT_SIG_SCHEME inScheme;
// TPMT_TK_CREATION creationTicket;
//
} TPM20_CertifyCreation_In;

typedef struct {
  TPM_ST tag;
  UINT32 responseSize;
  UINT32 responseCode; 
  UINT8 otherData;
//
// NOTE: other output data consists of the following fields.  But it can't be
// specified here because some structures can vary in size or may not
// even be present.
//
// TPM2B_ATTEST certifyInfo;
// TPMT_SIGNATURE signature;
//
// TPM_CMD_SESSION_DATA_OUT[] sessions;
//
} TPM20_CertifyCreation_Out;


//
// TPM20 Quote input/output structures
//
typedef struct {
  TPM_ST tag;
  UINT32 commandSize;
  TPM_CC commandCode;
  TPMI_DH_OBJECT signHandle;
  UINT8 otherData;
//
// NOTE: other input data consists of the following fields.  But it can't be
// specified here because some structures can vary in size.
//
//  TPM_CMD_SESSION_DATA_IN[];
//
//  TPM2B_DATA qualifyingData;
//  TPMT_SIG_SCHEME inScheme;
//  TPML_PCR_SELECTION pcrSelect;
//
} TPM20_Quote_In;

typedef struct {
  TPM_ST tag;
  UINT32 responseSize;
  UINT32 responseCode;
  UINT8 otherData;
//
// NOTE: other output data consists of the following fields.  But it can't be
// specified here because some structures can vary in size or may not
// even be present.
//
//  TPM2B_ATTEST quoted;
//  TPMT_SIGNATURE signature;
//
// TPM_CMD_SESSION_DATA_OUT[] sessions;
//
} TPM20_Quote_Out;


//
// TPM20 GetSessionAuditDigest input/output structures
//
typedef struct  {
  TPM_ST tag;
  UINT32 commandSize;
  TPM_CC commandCode;
  TPMI_RH_ENDORSEMENT privacyAdminHandle;
  TPMI_DH_OBJECT signHandle;
  TPMI_SH_HMAC sessionHandle;
  UINT8 otherData;
//
// NOTE: other input data consists of the following fields.  But it can't be
// specified here because some structures can vary in size.
//
// TPM_CMD_SESSION_DATA_IN[];
//
// TPM2B_DATA qualifyingData;
// TPMT_SIG_SCHEME inScheme;
//
} TPM20_GetSessionAuditDigest_In;

typedef struct {
  TPM_ST tag;
  UINT32 responseSize;
  UINT32 responseCode; 
  UINT8 otherData;
//
// NOTE: other output data consists of the following fields.  But it can't be
// specified here because some structures can vary in size or may not
// even be present.
//
// TPM2B_ATTEST auditInfo;
// TPMT_SIGNATURE signature;
//
// TPM_CMD_SESSION_DATA_OUT[] sessions;
//
} TPM20_GetSessionAuditDigest_Out;


//
// TPM20 GetCommandAuditDigest input/output structures
//
typedef struct  {
  TPM_ST tag;
  UINT32 commandSize;
  TPM_CC commandCode;
  TPMI_RH_ENDORSEMENT privacyHandle;
  TPMI_DH_OBJECT signHandle;
  UINT8 otherData;
//
// NOTE: other input data consists of the following fields.  But it can't be
// specified here because some structures can vary in size.
//
// TPM_CMD_SESSION_DATA_IN[];
//
// TPM2B_DATA qualifyingData;
// TPMT_SIG_SCHEME inScheme;
//
} TPM20_GetCommandAuditDigest_In;

typedef struct {
  TPM_ST tag;
  UINT32 responseSize;
  UINT32 responseCode; 
  UINT8 otherData;
//
// NOTE: other output data consists of the following fields.  But it can't be
// specified here because some structures can vary in size or may not
// even be present.
//
// TPM2B_ATTEST auditInfo;
// TPMT_SIGNATURE signature;
//
// TPM_CMD_SESSION_DATA_OUT[] sessions;
//
} TPM20_GetCommandAuditDigest_Out;


//
// TPM20 GetTime input/output structures
//
typedef struct  {
  TPM_ST tag;
  UINT32 commandSize;
  TPM_CC commandCode;
  TPMI_RH_ENDORSEMENT privacyAdminHandle;
  TPMI_DH_OBJECT signHandle;
  UINT8 otherData;
//
// NOTE: other input data consists of the following fields.  But it can't be
// specified here because some structures can vary in size.
//
// TPM_CMD_SESSION_DATA_IN[];
//
// TPM2B_ATTEST timeInfo;
// TPMT_SIG_SCHEME inScheme;
//
} TPM20_GetTime_In;

typedef struct {
  TPM_ST tag;
  UINT32 responseSize;
  UINT32 responseCode; 
  UINT8 otherData;
//
// NOTE: other output data consists of the following fields.  But it can't be
// specified here because some structures can vary in size or may not
// even be present.
//
// TPM2B_ATTEST auditInfo;
// TPMT_SIGNATURE signature;
//
// TPM_CMD_SESSION_DATA_OUT[] sessions;
//
} TPM20_GetTime_Out;


//
// TPM20 Commit input/output structures
//
typedef struct  {
  TPM_ST tag;
  UINT32 commandSize;
  TPM_CC commandCode;
  TPMI_DH_OBJECT keyHandle;
  UINT8 otherData;
//
// NOTE: other input data consists of the following fields.  But it can't be
// specified here because some structures can vary in size.
//
// TPM_CMD_SESSION_DATA_IN[];
//
// TPM2B_ECC_POINT P1;
// TPM2B_SENSISTIVE_DATA s2;
// TPM2B_ECC_PARAMETER y2;
//
} TPM20_Commit_In;

typedef struct {
  TPM_ST tag;
  UINT32 responseSize;
  UINT32 responseCode; 
  UINT8 otherData;
//
// NOTE: other output data consists of the following fields.  But it can't be
// specified here because some structures can vary in size or may not
// even be present.
//
// TPM2B_ECC_POINT K;
// TPM2B_ECC_POINT L;
// TPM2B_ECC_POINT E;
// UINT16 counter;
//
// TPM_CMD_SESSION_DATA_OUT[] sessions;
//
} TPM20_Commit_Out;


//
// TPM20 VerifySignature input/output structures
//
typedef struct  {
  TPM_ST tag;
  UINT32 commandSize;
  TPM_CC commandCode;
  TPMI_DH_OBJECT keyHandle;
  UINT8 otherData;
//
// NOTE: other input data consists of the following fields.  But it can't be
// specified here because some structures can vary in size.
//
// TPM_CMD_SESSION_DATA_IN[];
//
// TPM2B_DIGEST digest;
// TPMT_SIGNATURE signature;
//
} TPM20_VerifySignature_In;

typedef struct {
  TPM_ST tag;
  UINT32 responseSize;
  UINT32 responseCode; 
  UINT8 otherData;
//
// NOTE: other output data consists of the following fields.  But it can't be
// specified here because some structures can vary in size or may not
// even be present.
//
// TPMT_TK_VERIFIED validation;
//
// TPM_CMD_SESSION_DATA_OUT[] sessions;
//
} TPM20_VerifySignature_Out;


//
// TPM20 Sign input/output structures
//
typedef struct  {
  TPM_ST tag;
  UINT32 commandSize;
  TPM_CC commandCode;
  TPMI_DH_OBJECT keyHandle;
  UINT8 otherData;
//
// NOTE: other input data consists of the following fields.  But it can't be
// specified here because some structures can vary in size.
//
// TPM_CMD_SESSION_DATA_IN[];
//
// TPM2B_DIGEST digest;
// TPMT_SIG_SCHEME inScheme;
// TPMT_TK_HASHCHECK validation;
//
} TPM20_Sign_In;

typedef struct {
  TPM_ST tag;
  UINT32 responseSize;
  UINT32 responseCode; 
  UINT8 otherData;
//
// NOTE: other output data consists of the following fields.  But it can't be
// specified here because some structures can vary in size or may not
// even be present.
//
// TPMT_SIGNATURE signature;
//
// TPM_CMD_SESSION_DATA_OUT[] sessions;
//
} TPM20_Sign_Out;

//
// TPM20 SetCommandCodeAuditStatus input/output structures
//
typedef struct  {
  TPM_ST tag;
  UINT32 commandSize;
  TPM_CC commandCode;
  TPMI_RH_PROVISION auth;
  UINT8 otherData;
//
// NOTE: other input data consists of the following fields.  But it can't be
// specified here because some structures can vary in size.
//
// TPM_CMD_SESSION_DATA_IN[];
//
// TPMI_ALG_HASH auditAlg;
// TPML_CC setList;
// TPML_CC clearList;
//
} TPM20_SetCommandCodeAuditStatus_In;

typedef struct {
  TPM_ST tag;
  UINT32 responseSize;
  UINT32 responseCode; 
  UINT8 otherData;
//
// NOTE: other output data consists of the following fields.  But it can't be
// specified here because some structures can vary in size or may not
// even be present.
//
// TPM_CMD_SESSION_DATA_OUT[] sessions;
//
} TPM20_SetCommandCodeAuditStatus_Out;


//
// TPM20 PCR_Extend input/output structures
//
typedef struct {
  TPM_ST tag;
  UINT32 commandSize;
  TPM_CC commandCode;
  TPMI_DH_PCR pcrHandle;
  UINT8 otherData;
//
// NOTE: other input data consists of the following fields.  But it can't be
// specified here because some structures can vary in size.
//
//  TPM_CMD_SESSION_DATA_IN[];
//
//  TPML_DIGEST_VALUES digests;
//
} TPM20_PCR_Extend_In;

typedef struct {
  TPM_ST tag;
  UINT32 responseSize;
  UINT32 responseCode;
  UINT8 otherData;
//
// NOTE: other output data consists of the following fields.  But it can't be
// specified here because some structures can vary in size or may not
// even be present.
//
// TPM_CMD_SESSION_DATA_OUT[] sessions;
//
} TPM20_PCR_Extend_Out;



//
// TPM20 PCR_Event input/output structures
//
typedef struct {
  TPM_ST tag;
  UINT32 commandSize;
  TPM_CC commandCode;
  TPMI_DH_PCR pcrHandle;
  UINT8 otherData;
//
// NOTE: other input data consists of the following fields.  But it can't be
// specified here because some structures can vary in size.
//
//  TPM_CMD_SESSION_DATA_IN[];
//
//  TPM2B_EVENT eventData;
//
} TPM20_PCR_Event_In;

typedef struct {
  TPM_ST tag;
  UINT32 responseSize;
  UINT32 responseCode;
  UINT8 otherData;
//
// NOTE: other output data consists of the following fields.  But it can't be
// specified here because some structures can vary in size or may not
// even be present.
//
// TPML_DIGEST_VALUES digest;
//
// TPM_CMD_SESSION_DATA_OUT[] sessions;
//
} TPM20_PCR_Event_Out;


//
// TPM20 PCR_Read input/output structures
//
typedef struct {
  TPM_ST tag;
  UINT32 commandSize;
  TPM_CC commandCode;
  UINT8 otherData;
//
// NOTE: other input data consists of the following fields.  But it can't be
// specified here because some structures can vary in size.
//
//  TPM_CMD_SESSION_DATA_IN[];
//
//  TPML_PCR_SELECTION pcrSelection;
//
} TPM20_PCR_Read_In;

typedef struct {
  TPM_ST tag;
  UINT32 responseSize;
  UINT32 responseCode;
  UINT8 otherData;
//
// NOTE: other output data consists of the following fields.  But it can't be
// specified here because some structures can vary in size or may not
// even be present.
//
// TPM_CMD_SESSION_DATA_OUT[] sessions;
//
} TPM20_PCR_Read_Out;


//
// TPM20 PCR_Allocate input/output structures
//
typedef struct {
  TPM_ST tag;
  UINT32 commandSize;
  TPM_CC commandCode;
  TPMI_RH_PLATFORM authHandle;
  UINT8 otherData;
//
// NOTE: other input data consists of the following fields.  But it can't be
// specified here because some structures can vary in size.
//
//  TPM_CMD_SESSION_DATA_IN[];
//
//  TPML_PCR_SELECTION pcrAllocation;
//
} TPM20_PCR_Allocate_In;

#define SET_PCR_SELECT_BIT( pcrSelection, pcr ) \
(pcrSelection).pcrSelect[( (pcr)/8 )] |= ( 1 << ( (pcr) % 8) );

#define CLEAR_PCR_SELECT_BITS( pcrSelection ) \
(pcrSelection).pcrSelect[0] = 0; \
(pcrSelection).pcrSelect[1] = 0; \
(pcrSelection).pcrSelect[2] = 0;

#define SET_PCR_SELECT_SIZE( pcrSelection, size ) \
(pcrSelection).sizeofSelect = size; 


typedef struct {
  TPM_ST tag;
  UINT32 responseSize;
  UINT32 responseCode;
  UINT8 otherData;
//
// NOTE: other output data consists of the following fields.  But it can't be
// specified here because some structures can vary in size or may not
// even be present.
//
// TPM_CMD_SESSION_DATA_OUT[] sessions;
//
} TPM20_PCR_Allocate_Out;


//
// TPM20 PCR_SetAuthPolicy input/output structures
//
typedef struct  {
  TPM_ST tag;
  UINT32 commandSize;
  TPM_CC commandCode;
  TPMI_RH_PLATFORM authHandle;
  UINT8 otherData;
//
// NOTE: other input data consists of the following fields.  But it can't be
// specified here because some structures can vary in size.
//
// TPM_CMD_SESSION_DATA_IN[];
//
// TPM2B_DIGEST authPolicy;
// TPMI_ALG_HASH policyHash;
// TPMI_DH_PCR pcrNum;
//
} TPM20_PCR_SetAuthPolicy_In;

typedef struct {
  TPM_ST tag;
  UINT32 responseSize;
  UINT32 responseCode; 
  UINT8 otherData;
//
// NOTE: other output data consists of the following fields.  But it can't be
// specified here because some structures can vary in size or may not
// even be present.
//
// TPM_CMD_SESSION_DATA_OUT[] sessions;
//
} TPM20_PCR_SetAuthPolicy_Out;


//
// TPM20 PCR_SetAuthValue input/output structures
//
typedef struct  {
  TPM_ST tag;
  UINT32 commandSize;
  TPM_CC commandCode;
  TPMI_DH_PCR pcrHandle;
  UINT8 otherData;
//
// NOTE: other input data consists of the following fields.  But it can't be
// specified here because some structures can vary in size.
//
// TPM_CMD_SESSION_DATA_IN[];
//
// TPM2B_DIGEST auth;
//
} TPM20_PCR_SetAuthValue_In;

typedef struct {
  TPM_ST tag;
  UINT32 responseSize;
  UINT32 responseCode; 
  UINT8 otherData;
//
// NOTE: other output data consists of the following fields.  But it can't be
// specified here because some structures can vary in size or may not
// even be present.
//
// TPM_CMD_SESSION_DATA_OUT[] sessions;
//
} TPM20_PCR_SetAuthValue_Out;


//
// TPM20 PCR_Reset input/output structures
//
typedef struct  {
  TPM_ST tag;
  UINT32 commandSize;
  TPM_CC commandCode;
  TPMI_DH_PCR pcrHandle;
  UINT8 otherData;
//
// NOTE: other input data consists of the following fields.  But it can't be
// specified here because some structures can vary in size.
//
// TPM_CMD_SESSION_DATA_IN[];
//
} TPM20_PCR_Reset_In;

typedef struct {
  TPM_ST tag;
  UINT32 responseSize;
  UINT32 responseCode; 
  UINT8 otherData;
//
// NOTE: other output data consists of the following fields.  But it can't be
// specified here because some structures can vary in size or may not
// even be present.
//
// TPM_CMD_SESSION_DATA_OUT[] sessions;
//
} TPM20_PCR_Reset_Out;


//
// TPM20 PolicySigned input/output structures
//
typedef struct  {
  TPM_ST tag;
  UINT32 commandSize;
  TPM_CC commandCode;
  TPMI_DH_OBJECT authObject;
  TPMI_SH_POLICY policySession;
  UINT8 otherData;
//
// NOTE: other input data consists of the following fields.  But it can't be
// specified here because some structures can vary in size.
//
// TPM_CMD_SESSION_DATA_IN[];
//
// TPM2B_NONCE nonceTPM;
// TPM2B_DIGEST cpHashA;
// TPM2B_NONCE policyRef;
// UINT32 expiration;
// TPMT_SIGNATURE auth;
//
} TPM20_PolicySigned_In;

typedef struct {
  TPM_ST tag;
  UINT32 responseSize;
  UINT32 responseCode; 
  UINT8 otherData;
//
// NOTE: other output data consists of the following fields.  But it can't be
// specified here because some structures can vary in size or may not
// even be present.
//
// TPM2B_TIMEOUT timeout;
// TPMT_TK_AUTH policyTicket;
//
// TPM_CMD_SESSION_DATA_OUT[] sessions;
//
} TPM20_PolicySigned_Out;


//
// TPM20 PolicySecret input/output structures
//
typedef struct  {
  TPM_ST tag;
  UINT32 commandSize;
  TPM_CC commandCode;
  TPMI_DH_ENTITY authHandle;
  TPMI_SH_POLICY policySession;
  UINT8 otherData;
//
// NOTE: other input data consists of the following fields.  But it can't be
// specified here because some structures can vary in size.
//
// TPM_CMD_SESSION_DATA_IN[];
//
// TPM2B_NONCE nonceTPM;
// TPM2B_DIGEST cpHashA;
// TPM2B_NONCE policyRef;
// UINT32 expiration;
//
} TPM20_PolicySecret_In;

typedef struct {
  TPM_ST tag;
  UINT32 responseSize;
  UINT32 responseCode; 
  UINT8 otherData;
//
// NOTE: other output data consists of the following fields.  But it can't be
// specified here because some structures can vary in size or may not
// even be present.
//
// TPM2B_TIMEOUT timeout;
// TPMT_TK_AUTH policyTicket;
//
// TPM_CMD_SESSION_DATA_OUT[] sessions;
//
} TPM20_PolicySecret_Out;


//
// TPM20 PolicyTicket input/output structures
//
typedef struct  {
  TPM_ST tag;
  UINT32 commandSize;
  TPM_CC commandCode;
  TPMI_DH_OBJECT authObject;
  TPMI_SH_POLICY policySession;
  UINT8 otherData;
//
// NOTE: other input data consists of the following fields.  But it can't be
// specified here because some structures can vary in size.
//
// TPM_CMD_SESSION_DATA_IN[];
//
// TPM2B_TIMEOUT timeout;
// TPM2B_DIGEST cpHashA;
// TPM2B_NONCE policyRef;
// TPMT_SIGNATURE authName;
// TPMT_TK_AUTH ticket;
//
} TPM20_PolicyTicket_In;

typedef struct {
  TPM_ST tag;
  UINT32 responseSize;
  UINT32 responseCode; 
  UINT8 otherData;
//
// NOTE: other output data consists of the following fields.  But it can't be
// specified here because some structures can vary in size or may not
// even be present.
//
// TPM_CMD_SESSION_DATA_OUT[] sessions;
//
} TPM20_PolicyTicket_Out;


//
// TPM20 PolicyOR input/output structures
//
typedef struct  {
  TPM_ST tag;
  UINT32 commandSize;
  TPM_CC commandCode;
  TPMI_SH_POLICY policySession;
  UINT8 otherData;
//
// NOTE: other input data consists of the following fields.  But it can't be
// specified here because some structures can vary in size.
//
// TPM_CMD_SESSION_DATA_IN[];
//
// TPML_DIGEST pHashList;
//
} TPM20_PolicyOR_In;

typedef struct {
  TPM_ST tag;
  UINT32 responseSize;
  UINT32 responseCode; 
  UINT8 otherData;
//
// NOTE: other output data consists of the following fields.  But it can't be
// specified here because some structures can vary in size or may not
// even be present.
//
// TPM_CMD_SESSION_DATA_OUT[] sessions;
//
} TPM20_PolicyOR_Out;


//
// TPM20 PolicyPCR input/output structures
//
typedef struct {
  TPM_ST tag;
  UINT32 commandSize;
  TPM_CC commandCode;
  TPMI_SH_POLICY policySession;
  UINT8 otherData;
//
// NOTE: other input data consists of the following fields.  But it can't be
// specified here because some structures can vary in size.
//
//  TPM_CMD_SESSION_DATA_IN[];
//
//  TPM2B_DIGEST &pcrDigest;
//  TPML_PCR_SELECTION &pcrs;
//
} TPM20_PolicyPCR_In;

typedef struct {
  TPM_ST tag;
  UINT32 responseSize;
  UINT32 responseCode;
  UINT8 otherData;
//
// NOTE: other output data consists of the following fields.  But it can't be
// specified here because some structures can vary in size or may not
// even be present.
//
// TPM_CMD_SESSION_DATA_OUT[] sessions;
//
} TPM20_PolicyPCR_Out;



//
// TPM20 PolicyLocality input/output structures
//
typedef struct {
  TPM_ST tag;
  UINT32 commandSize;
  TPM_CC commandCode;
  TPMI_SH_POLICY policySession;
  UINT8 otherData;
//
// NOTE: other input data consists of the following fields.  But it can't be
// specified here because some structures can vary in size.
//
//  TPM_CMD_SESSION_DATA_IN[];
//
//  TPMA_LOCALITY locality;
//
} TPM20_PolicyLocality_In;

typedef struct {
  TPM_ST tag;
  UINT32 responseSize;
  UINT32 responseCode;
  UINT8 otherData;
//
// NOTE: other output data consists of the following fields.  But it can't be
// specified here because some structures can vary in size or may not
// even be present.
//
// TPM_CMD_SESSION_DATA_OUT[] sessions;
//
} TPM20_PolicyLocality_Out;



//
// TPM20 PolicyNV input/output structures
//
typedef struct {
  TPM_ST tag;
  UINT32 commandSize;
  TPM_CC commandCode;
  TPMI_RH_NV_AUTH authHandle;
  TPMI_RH_NV_INDEX nvIndex;
  TPMI_SH_POLICY policySession;
  UINT8 otherData;
//
// NOTE: other output data consists of the following fields.  But it can't be
// specified here because some structures can vary in size or may not
// even be present.
//
//  TPM2B_OPERAND operandB;
//  UINT16 offset;
//  TPM_EO operation;
//
} TPM20_PolicyNV_In;

typedef struct {
  TPM_ST tag;
  UINT32 responseSize;
  UINT32 responseCode;
  UINT8 otherData;
//
// NOTE: other output data consists of the following fields.  But it can't be
// specified here because some structures can vary in size or may not
// even be present.
//
// TPM_CMD_SESSION_DATA_OUT[] sessions;
//
//
} TPM20_PolicyNV_Out;


//
// TPM20 PolicyCounterTimer input/output structures
//
typedef struct  {
  TPM_ST tag;
  UINT32 commandSize;
  TPM_CC commandCode;
  TPMI_SH_POLICY policySession;
  UINT8 otherData;
//
// NOTE: other input data consists of the following fields.  But it can't be
// specified here because some structures can vary in size.
//
// TPM_CMD_SESSION_DATA_IN[];
//
// TPM2B_OPERAND operandB;
// UINT16 offset;
// TPM_EO operation;
//
} TPM20_PolicyCounterTimer_In;

typedef struct {
  TPM_ST tag;
  UINT32 responseSize;
  UINT32 responseCode; 
  UINT8 otherData;
//
// NOTE: other output data consists of the following fields.  But it can't be
// specified here because some structures can vary in size or may not
// even be present.
//
// TPM_CMD_SESSION_DATA_OUT[] sessions;
//
} TPM20_PolicyCounterTimer_Out;


//
// TPM20 PolicyCommandCode input/output structures
//
typedef struct {
  TPM_ST tag;
  UINT32 commandSize;
  TPM_CC commandCode;
  TPMI_SH_POLICY policySession;
  UINT8 otherData;
//
// NOTE: other input data consists of the following fields.  But it can't be
// specified here because some structures can vary in size.
//
// TPM_CMD_SESSION_DATA_IN[];
//
// TPM_CC code;
//
} TPM20_PolicyCommandCode_In;

typedef struct {
  TPM_ST tag;
  UINT32 responseSize;
  UINT32 responseCode;
  UINT8 otherData;
//
// NOTE: other output data consists of the following fields.  But it can't be
// specified here because some structures can vary in size or may not
// even be present.
//
// TPM_CMD_SESSION_DATA_OUT[] sessions;
//
} TPM20_PolicyCommandCode_Out;


//
// TPM20 PolicyPhysicalPresence input/output structures
//
typedef struct {
  TPM_ST tag;
  UINT32 commandSize;
  TPM_CC commandCode;
  TPMI_SH_POLICY policySession;
  UINT8 otherData;
//
// NOTE: other input data consists of the following fields.  But it can't be
// specified here because some structures can vary in size.
//
// TPM_CMD_SESSION_DATA_IN[];
//
} TPM20_PolicyPhysicalPresence_In;

typedef struct {
  TPM_ST tag;
  UINT32 responseSize;
  UINT32 responseCode;
  UINT8 otherData;
//
// NOTE: other output data consists of the following fields.  But it can't be
// specified here because some structures can vary in size or may not
// even be present.
//
// TPM_CMD_SESSION_DATA_OUT[] sessions;
//
} TPM20_PolicyPhysicalPresence_Out;


//
// TPM20 PolicyCpHash input/output structures
//
typedef struct {
  TPM_ST tag;
  UINT32 commandSize;
  TPM_CC commandCode;
  TPMI_SH_POLICY policySession;
  UINT8 otherData;
//
// NOTE: other input data consists of the following fields.  But it can't be
// specified here because some structures can vary in size.
//
// TPM_CMD_SESSION_DATA_IN[];
//
// TPM2B_DIGEST cpHashA;
//
} TPM20_PolicyCpHash_In;

typedef struct {
  TPM_ST tag;
  UINT32 responseSize;
  UINT32 responseCode;
  UINT8 otherData;
//
// NOTE: other output data consists of the following fields.  But it can't be
// specified here because some structures can vary in size or may not
// even be present.
//
// TPM_CMD_SESSION_DATA_OUT[] sessions;
//
} TPM20_PolicyCpHash_Out;


//
// TPM20 PolicyNameHash input/output structures
//
typedef struct {
  TPM_ST tag;
  UINT32 commandSize;
  TPM_CC commandCode;
  TPMI_SH_POLICY policySession;
  UINT8 otherData;
//
// NOTE: other input data consists of the following fields.  But it can't be
// specified here because some structures can vary in size.
//
// TPM_CMD_SESSION_DATA_IN[];
//
// TPM2B_DIGEST nameHash;
//
} TPM20_PolicyNameHash_In;

typedef struct {
  TPM_ST tag;
  UINT32 responseSize;
  UINT32 responseCode;
  UINT8 otherData;
//
// NOTE: other output data consists of the following fields.  But it can't be
// specified here because some structures can vary in size or may not
// even be present.
//
// TPM_CMD_SESSION_DATA_OUT[] sessions;
//
} TPM20_PolicyNameHash_Out;


//
// TPM20 PolicyDuplicationSelect input/output structures
//
typedef struct {
  TPM_ST tag;
  UINT32 commandSize;
  TPM_CC commandCode;
  TPMI_SH_POLICY policySession;
  UINT8 otherData;
//
// NOTE: other input data consists of the following fields.  But it can't be
// specified here because some structures can vary in size.
//
// TPM_CMD_SESSION_DATA_IN[];
//
// TPM2B_NAME objectName;
// TPM2B_NAME newParentName;
// TPMI_YES_NO includeObject;
//
} TPM20_PolicyDuplicationSelect_In;

typedef struct {
  TPM_ST tag;
  UINT32 responseSize;
  UINT32 responseCode;
  UINT8 otherData;
//
// NOTE: other output data consists of the following fields.  But it can't be
// specified here because some structures can vary in size or may not
// even be present.
//
// TPM_CMD_SESSION_DATA_OUT[] sessions;
//
} TPM20_PolicyDuplicationSelect_Out;


//
// TPM20 PolicyAuthorize input/output structures
//
typedef struct  {
  TPM_ST tag;
  UINT32 commandSize;
  TPM_CC commandCode;
  TPMI_SH_POLICY policySession;
  UINT8 otherData;
//
// NOTE: other input data consists of the following fields.  But it can't be
// specified here because some structures can vary in size.
//
// TPM_CMD_SESSION_DATA_IN[];
//
// TPM2B_DIGEST approvedPolicy;
// TPM2B_NONCE  policyRef;
// TPM2B_NAME keySign;
// TPMT_TK_VERIFIED checkTicket;
//
} TPM20_PolicyAuthorize_In;

typedef struct {
  TPM_ST tag;
  UINT32 responseSize;
  UINT32 responseCode; 
  UINT8 otherData;
//
// NOTE: other output data consists of the following fields.  But it can't be
// specified here because some structures can vary in size or may not
// even be present.
//
// TPM_CMD_SESSION_DATA_OUT[] sessions;
//
} TPM20_PolicyAuthorize_Out;


//
// TPM20 PolicyAuthValue input/output structures
//
typedef struct  {
  TPM_ST tag;
  UINT32 commandSize;
  TPM_CC commandCode;
  TPMI_SH_POLICY policySession;
  UINT8 otherData;
//
// NOTE: other input data consists of the following fields.  But it can't be
// specified here because some structures can vary in size.
//
// TPM_CMD_SESSION_DATA_IN[];
//
} TPM20_PolicyAuthValue_In;

typedef struct {
  TPM_ST tag;
  UINT32 responseSize;
  UINT32 responseCode; 
  UINT8 otherData;
//
// NOTE: other output data consists of the following fields.  But it can't be
// specified here because some structures can vary in size or may not
// even be present.
//
// TPM_CMD_SESSION_DATA_OUT[] sessions;
//
} TPM20_PolicyAuthValue_Out;


//
// TPM20 PolicyPassword input/output structures
//
typedef struct  {
  TPM_ST tag;
  UINT32 commandSize;
  TPM_CC commandCode;
  TPMI_SH_POLICY policySession;
  UINT8 otherData;
//
// NOTE: other input data consists of the following fields.  But it can't be
// specified here because some structures can vary in size.
//
// TPM_CMD_SESSION_DATA_IN[];
//
} TPM20_PolicyPassword_In;

typedef struct {
  TPM_ST tag;
  UINT32 responseSize;
  UINT32 responseCode; 
  UINT8 otherData;
//
// NOTE: other output data consists of the following fields.  But it can't be
// specified here because some structures can vary in size or may not
// even be present.
//
// TPM_CMD_SESSION_DATA_OUT[] sessions;
//
} TPM20_PolicyPassword_Out;


//
// TPM20 PolicyGetDigest input/output structures
//
typedef struct {
  TPM_ST tag;
  UINT32 commandSize;
  TPM_CC commandCode;
  TPMI_SH_POLICY policySession;
  UINT8 otherData;
//
// NOTE: other input data consists of the following fields.  But it can't be
// specified here because some structures can vary in size.
//
//  TPM_CMD_SESSION_DATA_IN[];
//
} TPM20_PolicyGetDigest_In;

typedef struct {
  TPM_ST tag;
  UINT32 responseSize;
  UINT32 responseCode;
  UINT8 otherData;
//
// NOTE: other output data consists of the following fields.  But it can't be
// specified here because some structures can vary in size or may not
// even be present.
//
// TPM2B_DIGEST policyHash;
//
// TPM_CMD_SESSION_DATA_OUT[] sessions;
//
} TPM20_PolicyGetDigest_Out;


//
// TPM20 CreatePrimary input/output structures
//
typedef struct {
    TPM_ST tag;
    UINT32 commandSize;
    TPM_CC commandCode;
    TPMI_RH_CLEAR primaryHandle;
    UINT8 otherData;
    //
    // NOTE: other input data consists of the following fields.  But it can't be
    // specified here because some structures can vary in size.
    //
    //  TPM_CMD_SESSION_DATA_IN[];
    //
    //  TPM2B_SENSITIVE_CREATE inSenstive;
    //  TPM2B_PUBLIC inPublic;
    //  TPM2B_DATA outsideInfo;
    //  TPML_PCR_SELECTION creationPCR;
    //
} TPM20_CreatePrimary_In;

typedef struct {
    TPM_ST tag;
    UINT32 responseSize;
    UINT32 responseCode;
    UINT8 otherData;
    //
    // NOTE: other output data consists of the following fields.  But it can't be
    // specified here because some structures can vary in size or may not
    // even be present.
    //
    //  TPM_HANDLE objectHandle;
    //
    // TPM2B_PUBLIC outPublic;
    // TPM2B_CREATION_DATA creationData;
    // TPM2B_DIGEST creationHash;
    // TPMT_TK_CREATION creationTicket;
    // TPM2B_NAME name;
    //
    // TPM_CMD_SESSION_DATA_OUT[] sessions;
    //
} TPM20_CreatePrimary_Out;


//
// TPM20 HierarchyControl input/output structures
//
typedef struct {
  TPM_ST tag;
  UINT32 commandSize;
  TPM_CC commandCode;
  TPMI_RH_CLEAR authHandle;
  UINT8 otherData;
//
// NOTE: other input data consists of the following fields.  But it can't be
// specified here because some structures can vary in size.
//
//  TPM_CMD_SESSION_DATA_IN[];
//
//  TPMI_RH_HIERARCHY hierarchy;
//  TPMI_YES_NO state;
//
} TPM20_HierarchyControl_In;

typedef struct {
  TPM_ST tag;
  UINT32 responseSize;
  UINT32 responseCode;
  UINT8 otherData;
//
// NOTE: other output data consists of the following fields.  But it can't be
// specified here because some structures can vary in size or may not
// even be present.
//
// TPM_CMD_SESSION_DATA_OUT[] sessions;
//
} TPM20_HierarchyControl_Out;


//
// TPM20 SetPrimaryPolicy input/output structures
//
typedef struct  {
  TPM_ST tag;
  UINT32 commandSize;
  TPM_CC commandCode;
  TPMI_RH_HIERARCHY authHandle;
  UINT8 otherData;
//
// NOTE: other input data consists of the following fields.  But it can't be
// specified here because some structures can vary in size.
//
// TPM_CMD_SESSION_DATA_IN[];
//
// TPM2B_DIGEST authPolicy;
// TPMI_ALG_HASH hashAlg;
//
} TPM20_SetPrimaryPolicy_In;

typedef struct {
  TPM_ST tag;
  UINT32 responseSize;
  UINT32 responseCode; 
  UINT8 otherData;
//
// NOTE: other output data consists of the following fields.  But it can't be
// specified here because some structures can vary in size or may not
// even be present.
//
// TPM_CMD_SESSION_DATA_OUT[] sessions;
//
} TPM20_SetPrimaryPolicy_Out;


//
// TPM20 ChangePPS input/output structures
//
typedef struct {
  TPM_ST tag;
  UINT32 commandSize;
  TPM_CC commandCode;
  TPMI_RH_PLATFORM authHandle;
  UINT8 otherData;
//
// NOTE: other input data consists of the following fields.  But it can't be
// specified here because some structures can vary in size.
//
//  TPM_CMD_SESSION_DATA_IN[];
//
} TPM20_ChangePPS_In;

typedef struct {
  TPM_ST tag;
  UINT32 responseSize;
  UINT32 responseCode;
  UINT8 otherData;
//
// NOTE: other output data consists of the following fields.  But it can't be
// specified here because some structures can vary in size or may not
// even be present.
//
// TPM_CMD_SESSION_DATA_OUT[] sessions;
//
} TPM20_ChangePPS_Out;


//
// TPM20 ChangeEPS input/output structures
//
typedef struct {
  TPM_ST tag;
  UINT32 commandSize;
  TPM_CC commandCode;
  TPMI_RH_PLATFORM authHandle;
  UINT8 otherData;
//
// NOTE: other input data consists of the following fields.  But it can't be
// specified here because some structures can vary in size.
//
//  TPM_CMD_SESSION_DATA_IN[];
//
} TPM20_ChangeEPS_In;

typedef struct {
  TPM_ST tag;
  UINT32 responseSize;
  UINT32 responseCode;
  UINT8 otherData;
//
// NOTE: other output data consists of the following fields.  But it can't be
// specified here because some structures can vary in size or may not
// even be present.
//
// TPM_CMD_SESSION_DATA_OUT[] sessions;
//
} TPM20_ChangeEPS_Out;


//
// TPM20 Clear input/output structures
//
typedef struct {
  TPM_ST tag;
  UINT32 commandSize;
  TPM_CC commandCode;
  TPMI_RH_CLEAR authHandle;
  UINT8 otherData;
//
// NOTE: other input data consists of the following fields.  But it can't be
// specified here because some structures can vary in size.
//
//  TPM_CMD_SESSION_DATA_IN[];
//
} TPM20_Clear_In;

typedef struct {
  TPM_ST tag;
  UINT32 responseSize;
  UINT32 responseCode;
  UINT8 otherData;
//
// NOTE: other output data consists of the following fields.  But it can't be
// specified here because some structures can vary in size or may not
// even be present.
//
// TPM_CMD_SESSION_DATA_OUT[] sessions;
//
} TPM20_Clear_Out;


//
// TPM20 ClearControl input/output structures
//
typedef struct {
  TPM_ST tag;
  UINT32 commandSize;
  TPM_CC commandCode;
  TPMI_RH_CLEAR auth;
  UINT8 otherData;
//
// NOTE: other input data consists of the following fields.  But it can't be
// specified here because some structures can vary in size.
//
//  TPM_CMD_SESSION_DATA_IN[];
//
} TPM20_ClearControl_In;

typedef struct {
  TPM_ST tag;
  UINT32 responseSize;
  UINT32 responseCode;
  UINT8 otherData;
//
// NOTE: other output data consists of the following fields.  But it can't be
// specified here because some structures can vary in size or may not
// even be present.
//
// TPM_CMD_SESSION_DATA_OUT[] sessions;
//
} TPM20_ClearControl_Out;


//
// TPM20 HierarchyChangeAuth input/output structures
//
typedef struct {
  TPM_ST tag;
  UINT32 commandSize;
  TPM_CC commandCode;
  TPMI_RH_HIERARCHY_AUTH authHandle;
  UINT8 otherData;
//
// NOTE: other input data consists of the following fields.  But it can't be
// specified here because some structures can vary in size.
//
//  TPM_CMD_SESSION_DATA_IN[];
//
} TPM20_HierarchyChangeAuth_In;

typedef struct {
  TPM_ST tag;
  UINT32 responseSize;
  UINT32 responseCode;
  UINT8 otherData;
//
// NOTE: other output data consists of the following fields.  But it can't be
// specified here because some structures can vary in size or may not
// even be present.
//
// TPM_CMD_SESSION_DATA_OUT[] sessions;
//
} TPM20_HierarchyChangeAuth_Out;


//
// TPM20 DictionaryAttackLockReset input/output structures
//
typedef struct  {
  TPM_ST tag;
  UINT32 commandSize;
  TPM_CC commandCode;
  TPMI_RH_LOCKOUT lockHandle;
  UINT8 otherData;
//
// NOTE: other input data consists of the following fields.  But it can't be
// specified here because some structures can vary in size.
//
//  TPM_CMD_SESSION_DATA_IN[];
//
} TPM20_DictionaryAttackLockReset_In;

typedef struct {
  TPM_ST tag;
  UINT32 responseSize;
  UINT32 responseCode; 
  UINT8 otherData;
//
// NOTE: other output data consists of the following fields.  But it can't be
// specified here because some structures can vary in size or may not
// even be present.
//
// TPM_CMD_SESSION_DATA_OUT[] sessions;
//
} TPM20_DictionaryAttackLockReset_Out;


//
// TPM20 DictionaryAttackParameters input/output structures
//
typedef struct  {
  TPM_ST tag;
  UINT32 commandSize;
  TPM_CC commandCode;
  TPMI_RH_LOCKOUT lockHandle;
  UINT8 otherData;
//
// NOTE: other input data consists of the following fields.  But it can't be
// specified here because some structures can vary in size.
//
//  TPM_CMD_SESSION_DATA_IN[];
//
// UINT32 newMaxTries;
// UINT32 newRecoveryTime;
// UINT32 lockoutRecovery;
//
} TPM20_DictionaryAttackParameters_In;

typedef struct {
  TPM_ST tag;
  UINT32 responseSize;
  UINT32 responseCode; 
  UINT8 otherData;
//
// NOTE: other output data consists of the following fields.  But it can't be
// specified here because some structures can vary in size or may not
// even be present.
//
// TPM_CMD_SESSION_DATA_OUT[] sessions;
//
} TPM20_DictionaryAttackParameters_Out;


//
// TPM20 PP_Commands input/output structures
//
typedef struct  {
  TPM_ST tag;
  UINT32 commandSize;
  TPM_CC commandCode;
  TPMI_RH_PLATFORM auth;
  UINT8 otherData;
//
// NOTE: other input data consists of the following fields.  But it can't be
// specified here because some structures can vary in size.
//
// TPM_CMD_SESSION_DATA_IN[];
//
// TPML_CC setList;
// TPML_CC clearList;
//
} TPM20_PP_Commands_In;

typedef struct {
  TPM_ST tag;
  UINT32 responseSize;
  UINT32 responseCode; 
  UINT8 otherData;
//
// NOTE: other output data consists of the following fields.  But it can't be
// specified here because some structures can vary in size or may not
// even be present.
//
// TPM_CMD_SESSION_DATA_OUT[] sessions;
//
} TPM20_PP_Commands_Out;


//
// TPM20 SetAlgorithmSet input/output structures
//
typedef struct  {
  TPM_ST tag;
  UINT32 commandSize;
  TPM_CC commandCode;
  TPMI_RH_PLATFORM authHandle;
  UINT8 otherData;
//
// NOTE: other input data consists of the following fields.  But it can't be
// specified here because some structures can vary in size.
//
// TPM_CMD_SESSION_DATA_IN[];
//
// UINT32 algorithmSet;
//
} TPM20_SetAlgorithmSet_In;

typedef struct {
  TPM_ST tag;
  UINT32 responseSize;
  UINT32 responseCode; 
  UINT8 otherData;
//
// NOTE: other output data consists of the following fields.  But it can't be
// specified here because some structures can vary in size or may not
// even be present.
//
// TPM_CMD_SESSION_DATA_OUT[] sessions;
//
} TPM20_SetAlgorithmSet_Out;


//
// TPM20 FieldUpgradeStart input/output structures
//
typedef struct  {
  TPM_ST tag;
  UINT32 commandSize;
  TPM_CC commandCode;
  TPMI_RH_PLATFORM authorization;
  TPMI_DH_OBJECT keyHandle;
  UINT8 otherData;
//
// NOTE: other input data consists of the following fields.  But it can't be
// specified here because some structures can vary in size.
//
// TPM_CMD_SESSION_DATA_IN[];
//
// TPM2B_DIGEST fuDigest;
// TPMT_SIGNATURE manifestSignature;
//
} TPM20_FieldUpgradeStart_In;

typedef struct {
  TPM_ST tag;
  UINT32 responseSize;
  UINT32 responseCode; 
  UINT8 otherData;
//
// NOTE: other output data consists of the following fields.  But it can't be
// specified here because some structures can vary in size or may not
// even be present.
//
// TPM_CMD_SESSION_DATA_OUT[] sessions;
//
} TPM20_FieldUpgradeStart_Out;


//
// TPM20 FieldUpgradeData input/output structures
//
typedef struct  {
  TPM_ST tag;
  UINT32 commandSize;
  TPM_CC commandCode;
  UINT8 otherData;
//
// NOTE: other input data consists of the following fields.  But it can't be
// specified here because some structures can vary in size.
//
// TPM_CMD_SESSION_DATA_IN[];
//
// TPM2B_MAX_BUFFER fuData;
//
} TPM20_FieldUpgradeData_In;

typedef struct {
  TPM_ST tag;
  UINT32 responseSize;
  UINT32 responseCode; 
  UINT8 otherData;
//
// NOTE: other output data consists of the following fields.  But it can't be
// specified here because some structures can vary in size or may not
// even be present.
//
// TPMT_HA nextDigest;
// TPMT_HA firstDigest;
//
// TPM_CMD_SESSION_DATA_OUT[] sessions;
//
} TPM20_FieldUpgradeData_Out;


//
// TPM20 FirmwareRead input/output structures
//
typedef struct  {
  TPM_ST tag;
  UINT32 commandSize;
  TPM_CC commandCode;
  UINT8 otherData;
//
// NOTE: other input data consists of the following fields.  But it can't be
// specified here because some structures can vary in size.
//
// TPM_CMD_SESSION_DATA_IN[];
//
// UINT32 sequenceNumber;
//
} TPM20_FirmwareRead_In;

typedef struct {
  TPM_ST tag;
  UINT32 responseSize;
  UINT32 responseCode; 
  UINT8 otherData;
//
// NOTE: other output data consists of the following fields.  But it can't be
// specified here because some structures can vary in size or may not
// even be present.
//
// TPM2B_MAX_BUFFER fuData;
//
// TPM_CMD_SESSION_DATA_OUT[] sessions;
//
} TPM20_FirmwareRead_Out;


//
// TPM20 ContextSave input/output structures
//
typedef struct  {
  TPM_ST tag;
  UINT32 commandSize;
  TPM_CC commandCode;
  TPMI_DH_CONTEXT saveHandle;
} TPM20_ContextSave_In;

typedef struct {
  TPM_ST tag;
  UINT32 responseSize;
  UINT32 responseCode; 
  UINT8 otherData;
//
// NOTE: other output data consists of the following fields.  But it can't be
// specified here because some structures can vary in size or may not
// even be present.
//
// TPMS_CONTEXT context;
//
} TPM20_ContextSave_Out;


//
// TPM20 ContextLoad input/output structures
//
typedef struct  {
  TPM_ST tag;           
  UINT32 commandSize;
  TPM_CC commandCode;
  TPMS_CONTEXT *context;
} TPM20_ContextLoad_In;

typedef struct {
  TPM_ST tag;
  UINT32 responseSize;
  UINT32 responseCode; 
  UINT8 otherData;
//
// NOTE: other output data consists of the following fields.  But it can't be
// specified here because some structures can vary in size or may not
// even be present.
//
// TPMI_DH_CONTEXT loadedHandle;
//
} TPM20_ContextLoad_Out;




//
// TPM20 FlushContext input/output structures
//
typedef struct {
  TPM_ST tag;
  UINT32 commandSize;
  TPM_CC commandCode;
  TPMI_DH_CONTEXT flushHandle;
} TPM20_FlushContext_In;

typedef struct {
  TPM_ST tag;
  UINT32 responseSize;
  UINT32 responseCode;
} TPM20_FlushContext_Out;


//
// TPM20 EvictControl input/output structures
//
typedef struct  {
  TPM_ST tag;
  UINT32 commandSize;
  TPM_CC commandCode;
  TPMI_RH_PROVISION auth;
  TPMI_DH_OBJECT objectHandle;
  UINT8 otherData;
//
// NOTE: other input data consists of the following fields.  But it can't be
// specified here because some structures can vary in size.
//
// TPM_CMD_SESSION_DATA_IN[];
//
// TPMI_DH_PERSISTENT persistentHandle;
//
} TPM20_EvictControl_In;



typedef struct {
  TPM_ST tag;
  UINT32 responseSize;
  UINT32 responseCode; 
  UINT8 otherData;
//
// NOTE: other output data consists of the following fields.  But it can't be
// specified here because some structures can vary in size or may not
// even be present.
//
// TPM_CMD_SESSION_DATA_OUT[] sessions;
//
} TPM20_EvictControl_Out;


//
// TPM20 ReadClock input/output structures
//
typedef struct  {
  TPM_ST tag;
  UINT32 commandSize;
  TPM_CC commandCode;
  UINT8 otherData;
//
// NOTE: other input data consists of the following fields.  But it can't be
// specified here because some structures can vary in size.
//
// TPM_CMD_SESSION_DATA_IN[];
//
} TPM20_ReadClock_In;

typedef struct {
  TPM_ST tag;
  UINT32 responseSize;
  UINT32 responseCode; 
  UINT8 otherData;
//
// NOTE: other output data consists of the following fields.  But it can't be
// specified here because some structures can vary in size or may not
// even be present.
//
// TPMS_TIME_INFO currentTime;
//
// TPM_CMD_SESSION_DATA_OUT[] sessions;
//
} TPM20_ReadClock_Out;



//
// TPM20 ClockSet input/output structures
//
typedef struct  {
  TPM_ST tag;
  UINT32 commandSize;
  TPM_CC commandCode;
  TPMI_RH_PROVISION auth;
  UINT8 otherData;
//
// NOTE: other input data consists of the following fields.  But it can't be
// specified here because some structures can vary in size.
//
// TPM_CMD_SESSION_DATA_IN[];
//
// UINT64 newTime;
//
} TPM20_ClockSet_In;

typedef struct {
  TPM_ST tag;
  UINT32 responseSize;
  UINT32 responseCode; 
  UINT8 otherData;
//
// NOTE: other output data consists of the following fields.  But it can't be
// specified here because some structures can vary in size or may not
// even be present.
//
// TPM_CMD_SESSION_DATA_OUT[] sessions;
//
} TPM20_ClockSet_Out;


//
// TPM20 ClockRateAdjust input/output structures
//
typedef struct  {
  TPM_ST tag;
  UINT32 commandSize;
  TPM_CC commandCode;
  TPMI_RH_PROVISION auth;
  UINT8 otherData;
//
// NOTE: other input data consists of the following fields.  But it can't be
// specified here because some structures can vary in size.
//
// TPM_CMD_SESSION_DATA_IN[];
//
// TPM_CLOCK_ADJUST rateAdjust;
//
} TPM20_ClockRateAdjust_In;

typedef struct {
  TPM_ST tag;
  UINT32 responseSize;
  UINT32 responseCode; 
  UINT8 otherData;
//
// NOTE: other output data consists of the following fields.  But it can't be
// specified here because some structures can vary in size or may not
// even be present.
//
// TPM_CMD_SESSION_DATA_OUT[] sessions;
//
} TPM20_ClockRateAdjust_Out;


//
// TPM20 GetCapability input/output structures
//
typedef struct {
  TPM_ST tag;
  UINT32 commandSize;
  TPM_CC commandCode;
  UINT8 otherData;
//
// NOTE: other input data consists of the following fields.  But it can't be
// specified here because some structures can vary in size.
//
// TPM_CMD_SESSION_DATA_IN[];
//
// UINT8 moreData;
// TPMS_CAPABILITY_DATA capabilityData;
//
} TPM20_GetCapability_In;

typedef struct {
  TPM_ST tag;
  UINT32 responseSize;
  UINT32 responseCode;
  UINT8  otherData;
//
// NOTE: other output data consists of the following fields.  But it can't be
// specified here because some structures can vary in size or may not
// even be present.
//
// TPMI_YES_NO moreData;
// TPMS_CAPABILITY_DATA capabilityData;
//
// TPM_CMD_SESSION_DATA_OUT[] sessions;
//
} TPM20_GetCapability_Out;


//
// TPM20 TestParms input/output structures
//
typedef struct  {
  TPM_ST tag;
  UINT32 commandSize;
  TPM_CC commandCode;
  TPMT_PUBLIC_PARMS  parameters;
  UINT8 otherData;
//
// NOTE: other input data consists of the following fields.  But it can't be
// specified here because some structures can vary in size.
//
// TPM_CMD_SESSION_DATA_IN[];
//
} TPM20_TestParms_In;

typedef struct {
  TPM_ST tag;
  UINT32 responseSize;
  UINT32 responseCode; 
  UINT8 otherData;
//
// NOTE: other output data consists of the following fields.  But it can't be
// specified here because some structures can vary in size or may not
// even be present.
//
// TPM_CMD_SESSION_DATA_OUT[] sessions;
//
} TPM20_TestParms_Out;


//
// TPM20 NV_DefineSpace input/output structures
//
typedef struct {
  TPM_ST tag;
  UINT32 commandSize;
  TPM_CC commandCode;
  TPMI_RH_PROVISION authHandle;
  UINT8 otherData;
//
// NOTE: other input data consists of the following fields.  But it can't be
// specified here because some structures can vary in size.
//
//  TPM_CMD_SESSION_DATA_IN[];
//
//  TPM2B_AUTH auth;
//  TPM2B_NV_PUBLIC publicInfo;
//
  
} TPM20_NV_DefineSpace_In;

typedef struct {
  TPM_ST tag;
  UINT32 responseSize;
  UINT32 responseCode;
  UINT8 otherData;
//
// NOTE: other output data consists of the following fields.  But it can't be
// specified here because some structures can vary in size or may not
// even be present.
//
// TPM_CMD_SESSION_DATA_OUT[] sessions;
//
} TPM20_NV_DefineSpace_Out;


//
// TPM20 NV_UndefineSpace input/output structures
//
typedef struct {
  TPM_ST tag;
  UINT32 commandSize;
  TPM_CC commandCode;
  TPMI_RH_PROVISION authHandle;
  TPMI_RH_NV_INDEX nvIndex;
  UINT8 otherData;
//
// NOTE: other input data consists of the following fields.  But it can't be
// specified here because some structures can vary in size.
//
//  TPM_CMD_SESSION_DATA_IN[];
//
} TPM20_NV_UndefineSpace_In;

typedef struct {
  TPM_ST tag;
  UINT32 responseSize;
  UINT32 responseCode;
  UINT8 otherData;
//
// NOTE: other output data consists of the following fields.  But it can't be
// specified here because some structures can vary in size or may not
// even be present.
//
// TPM_CMD_SESSION_DATA_OUT[] sessions;
//
} TPM20_NV_UndefineSpace_Out;


//
// TPM20 NV_UndefineSpaceSpecial input/output structures
//
typedef struct {
  TPM_ST tag;
  UINT32 commandSize;
  TPM_CC commandCode;
  TPMI_RH_NV_INDEX nvIndex;
  TPMI_RH_PLATFORM platform;
  UINT8 otherData;
//
// NOTE: other input data consists of the following fields.  But it can't be
// specified here because some structures can vary in size.
//
//  Note:  first two sessions below are associated with
//  the nvIndex and platform handles.
//  TPM_CMD_SESSION_DATA_IN[];
//
} TPM20_NV_UndefineSpaceSpecial_In;

typedef struct {
  TPM_ST tag;
  UINT32 responseSize;
  UINT32 responseCode;
  UINT8 otherData;
//
// NOTE: other output data consists of the following fields.  But it can't be
// specified here because some structures can vary in size.
//
//  Note:  first two sessions below are associated with
//  the nvIndex and platform handles.
//
//  TPM_CMD_SESSION_DATA_IN[];
//
} TPM20_NV_UndefineSpaceSpecial_Out;


//
// TPM20 NV_ReadPublic input/output structures
//
typedef struct  {
  TPM_ST tag;
  UINT32 commandSize;
  TPM_CC commandCode;
  TPMI_RH_NV_INDEX nvIndex;
  UINT8 otherData;
//
// NOTE: other input data consists of the following fields.  But it can't be
// specified here because some structures can vary in size.
//
// TPM_CMD_SESSION_DATA_IN[];
//
} TPM20_NV_ReadPublic_In;

typedef struct {
  TPM_ST tag;
  UINT32 responseSize;
  UINT32 responseCode; 
  UINT8 otherData;
//
// NOTE: other output data consists of the following fields.  But it can't be
// specified here because some structures can vary in size or may not
// even be present.
//
// TPM2B_NV_PUBLIC nvPublic;
// TPM2B_NAME nvName;
//
// TPM_CMD_SESSION_DATA_OUT[] sessions;
//
} TPM20_NV_ReadPublic_Out;


//
// TPM20 NV_Write input/output structures
//
typedef struct {
  TPM_ST tag;
  UINT32 commandSize;
  TPM_CC commandCode;
  TPMI_RH_PROVISION authHandle;
  TPMI_RH_NV_INDEX nvIndex;
  UINT8 otherData;
//
// NOTE: other input data consists of the following fields.  But it can't be
// specified here because some structures can vary in size.
//
//  TPM_CMD_SESSION_DATA_IN[];
//
//  TPM2B_MAX_NV_BUFFER data;
//  UINT16 offset;
//
} TPM20_NV_Write_In;

typedef struct {
  TPM_ST tag;
  UINT32 responseSize;
  UINT32 responseCode;
  UINT8 otherData;
//
// NOTE: other output data consists of the following fields.  But it can't be
// specified here because some structures can vary in size or may not
// even be present.
//
// TPM2B_MAX_NV_BUFFER data;
//
// TPM_CMD_SESSION_DATA_OUT[] sessions;
//
} TPM20_NV_Write_Out;


//
// TPM20 NV_Increment input/output structures
//
typedef struct {
  TPM_ST tag;
  UINT32 commandSize;
  TPM_CC commandCode;
  TPMI_RH_NV_AUTH authHandle;
  TPMI_RH_NV_INDEX nvIndex;
  UINT8 otherData;
//
// NOTE: other input data consists of the following fields.  But it can't be
// specified here because some structures can vary in size.
//
//  TPM_CMD_SESSION_DATA_IN[];
//
} TPM20_NV_Increment_In;

typedef struct {
  TPM_ST tag;
  UINT32 responseSize;
  UINT32 responseCode;
  UINT8 otherData;
//
// NOTE: other output data consists of the following fields.  But it can't be
// specified here because some structures can vary in size or may not
// even be present.
//
// TPM_CMD_SESSION_DATA_OUT[] sessions;
//
} TPM20_NV_Increment_Out;


//
// TPM20 NV_Extend input/output structures
//
typedef struct  {
  TPM_ST tag;
  UINT32 commandSize;
  TPM_CC commandCode;
  TPMI_RH_NV_AUTH authHandle;
  TPMI_RH_NV_INDEX nvIndex;
  UINT8 otherData;
//
// NOTE: other input data consists of the following fields.  But it can't be
// specified here because some structures can vary in size.
//
// TPM_CMD_SESSION_DATA_IN[];
//
// TPM2B_MAX_NV_BUFFER data;
//
} TPM20_NV_Extend_In;

typedef struct {
  TPM_ST tag;
  UINT32 responseSize;
  UINT32 responseCode; 
  UINT8 otherData;
//
// NOTE: other output data consists of the following fields.  But it can't be
// specified here because some structures can vary in size or may not
// even be present.
//
// TPM_CMD_SESSION_DATA_OUT[] sessions;
//
} TPM20_NV_Extend_Out;


//
// TPM20 NV_SetBits input/output structures
//
typedef struct  {
  TPM_ST tag;
  UINT32 commandSize;
  TPM_CC commandCode;
  TPMI_RH_NV_AUTH authHandle;
  TPMI_RH_NV_INDEX nvIndex;
  UINT8 otherData;
//
// NOTE: other input data consists of the following fields.  But it can't be
// specified here because some structures can vary in size.
//
// TPM_CMD_SESSION_DATA_IN[];
//
} TPM20_NV_SetBits_In;

typedef struct {
  TPM_ST tag;
  UINT32 responseSize;
  UINT32 responseCode; 
  UINT8 otherData;
//
// NOTE: other output data consists of the following fields.  But it can't be
// specified here because some structures can vary in size or may not
// even be present.
//
// TPM_CMD_SESSION_DATA_OUT[] sessions;
//
} TPM20_NV_SetBits_Out;


//
// TPM20 NV_WriteLock input/output structures
//
typedef struct {
  TPM_ST tag;
  UINT32 commandSize;
  TPM_CC commandCode;
  TPMI_RH_PROVISION authHandle;
  TPMI_RH_NV_INDEX nvIndex;
  UINT8 otherData;
//
// NOTE: other input data consists of the following fields.  But it can't be
// specified here because some structures can vary in size.
//
//  TPM_CMD_SESSION_DATA_IN[];
//
} TPM20_NV_WriteLock_In;

typedef struct {
  TPM_ST tag;
  UINT32 responseSize;
  UINT32 responseCode;
  UINT8 otherData;
//
// NOTE: other output data consists of the following fields.  But it can't be
// specified here because some structures can vary in size or may not
// even be present.
//
// TPM2B_MAX_NV_BUFFER data;
//
// TPM_CMD_SESSION_DATA_OUT[] sessions;
//
} TPM20_NV_WriteLock_Out;


//
// TPM20 NV_GlobalWriteLock input/output structures
//
typedef struct {
  TPM_ST tag;
  UINT32 commandSize;
  TPM_CC commandCode;
  TPMI_RH_PROVISION authHandle;
  UINT8 otherData;
//
// NOTE: other input data consists of the following fields.  But it can't be
// specified here because some structures can vary in size.
//
//  TPM_CMD_SESSION_DATA_IN[];
//
} TPM20_NV_GlobalWriteLock_In;

typedef struct {
  TPM_ST tag;
  UINT32 responseSize;
  UINT32 responseCode;
  UINT8 otherData;
//
// NOTE: other output data consists of the following fields.  But it can't be
// specified here because some structures can vary in size or may not
// even be present.
//
// TPM_CMD_SESSION_DATA_OUT[] sessions;
//
} TPM20_NV_GlobalWriteLock_Out;


//
// TPM20 NV_Read input/output structures
//
typedef struct {
  TPM_ST tag;
  UINT32 commandSize;
  TPM_CC commandCode;
  TPMI_RH_PROVISION authHandle;
  TPMI_RH_NV_INDEX nvIndex;
  UINT8 otherData;
//
// NOTE: other input data consists of the following fields.  But it can't be
// specified here because some structures can vary in size.
//
//  TPM_CMD_SESSION_DATA_IN[];
//
//  UINT16 size;
//  UINT16 offset;
//
} TPM20_NV_Read_In;

typedef struct {
  TPM_ST tag;
  UINT32 responseSize;
  UINT32 responseCode;
  UINT8  otherData;
//
// NOTE: other output data consists of the following fields.  But it can't be
// specified here because some structures can vary in size or may not
// even be present.
//
//  TPM2B_MAX_NV_BUFFER data;
//
} TPM20_NV_Read_Out;


//
// TPM20 NV_ReadLock input/output structures
//
typedef struct {
  TPM_ST tag;
  UINT32 commandSize;
  TPM_CC commandCode;
  TPMI_RH_PROVISION authHandle;
  TPMI_RH_NV_INDEX nvIndex;
  UINT8 otherData;
//
// NOTE: other input data consists of the following fields.  But it can't be
// specified here because some structures can vary in size.
//
//  TPM_CMD_SESSION_DATA_IN[];
//
} TPM20_NV_ReadLock_In;

typedef struct {
  TPM_ST tag;
  UINT32 responseSize;
  UINT32 responseCode;
  UINT8 otherData;
//
// NOTE: other output data consists of the following fields.  But it can't be
// specified here because some structures can vary in size or may not
// even be present.
//
// TPM_CMD_SESSION_DATA_OUT[] sessions;
//
} TPM20_NV_ReadLock_Out;


//
// TPM20 NV_ChangeAuth input/output structures
//
typedef struct {
  TPM_ST tag;
  UINT32 commandSize;
  TPM_CC commandCode;
  TPMI_RH_NV_INDEX nvIndex;
  UINT8 otherData;
//
// NOTE: other input data consists of the following fields.  But it can't be
// specified here because some structures can vary in size.
//
//  TPM_CMD_SESSION_DATA_IN[];
//
// TPM2B_AUTH newAuth;
//
} TPM20_NV_ChangeAuth_In;

typedef struct {
  TPM_ST tag;
  UINT32 responseSize;
  UINT32 responseCode;
  UINT8 otherData;
//
// NOTE: other output data consists of the following fields.  But it can't be
// specified here because some structures can vary in size or may not
// even be present.
//
// TPM_CMD_SESSION_DATA_OUT[] sessions;
//
} TPM20_NV_ChangeAuth_Out;


//
// TPM20 NV_CertifyCommand input/output structures
//
typedef struct  {
  TPM_ST tag;
  UINT32 commandSize;
  TPM_CC commandCode;
  TPMI_DH_OBJECT signHandle;
  TPMI_RH_NV_AUTH authHandle;
  TPMI_RH_NV_INDEX nvIndex;
  UINT8 otherData;
//
// NOTE: other input data consists of the following fields.  But it can't be
// specified here because some structures can vary in size.
//
// TPM_CMD_SESSION_DATA_IN[];
//
// TPM2B_DATA qualifyingData;
// TPMT_SIG_SCHEME inScheme;
// UINT16 size;
// UINT16 offset;
//
} TPM20_NV_Certify_In;

typedef struct {
  TPM_ST tag;
  UINT32 responseSize;
  UINT32 responseCode; 
  UINT8 otherData;
//
// NOTE: other output data consists of the following fields.  But it can't be
// specified here because some structures can vary in size or may not
// even be present.
//
// TPM2B_ATTEST certifyInfo;
// TPMT_SIGNATURE signature;
//
// TPM_CMD_SESSION_DATA_OUT[] sessions;
//
} TPM20_NV_Certify_Out;




// Utility functions.
extern CPP_EXTERN void CopyCommandHeader( TPM_CC commandCode, TPM_CMD_SESSIONS_DATA_IN *pSessionDataIn );
extern void FinishCommand( void *otherData, UINT32 tabHandle, UINT32 tpmHandle, UINT32 *responseSize );

extern CPP_EXTERN UINT16 GetDigestSize( TPM_ALG_ID authHash );
extern CPP_EXTERN void CopySessionsDataIn( void **otherData, TPM_CMD_SESSIONS_DATA_IN *pSessionDataIn );
extern CPP_EXTERN void CopySessionDataIn( void **otherData, TPM_CMD_SESSION_DATA_IN *sessionData, UINT32 *sessionSizePtr );
extern CPP_EXTERN void CopySessionsDataOut( 
    TPM_CMD_SESSIONS_DATA_OUT *pSessionDataOut,
    void *otherData,
    TPM_ST tag, 
    TPM_CMD_SESSIONS_DATA_IN *pSessionDataIn
    );
extern CPP_EXTERN void CopySessionDataOut( TPM_CMD_SESSION_DATA_OUT *sessionData, void **otherData);
extern CPP_EXTERN UINT16 CopySizedByteBufferIn( TPM2B *dest, TPM2B *src );
extern CPP_EXTERN UINT16 CopySizedByteBufferOut( TPM2B *dest, TPM2B *src );
extern CPP_EXTERN void CopyPublicIn( void **otherData, TPM2B_PUBLIC *publicStr );
extern CPP_EXTERN void CopyTPMLDigest_LE_To_BE_Canonical( void **otherData, TPML_DIGEST *digestList );
extern CPP_EXTERN TPMI_SH_AUTH_SESSION StartPolicySession();
extern CPP_EXTERN void CreateNullSession( TPM_CMD_SESSION_DATA_IN *nullSessionData );
extern CPP_EXTERN TPMI_SH_AUTH_SESSION InitNvAuxPolicySession();
extern CPP_EXTERN void CopySensitive_LE_To_BE_Canonical( void **otherData, TPM2B_SENSITIVE *sensitive );
extern CPP_EXTERN void CopyCreationDataOut( TPM2B_CREATION_DATA *creationDataOut, void **otherData );
extern CPP_EXTERN void CopyPublicOut( TPM2B_PUBLIC *pub, void **otherData );
extern CPP_EXTERN void CopyTicketIn( void **otherData, TPMT_TK_CREATION *ticket );
extern CPP_EXTERN void CopyTicketOut( TPMT_TK_CREATION *ticket, void **otherData );
extern CPP_EXTERN void CopyDigestValuesIn( void **otherData, TPML_DIGEST_VALUES *pTpmlDigest );
extern CPP_EXTERN void CopyDigestValuesOut( TPML_DIGEST_VALUES *pTpmlDigest, void **otherData );
extern CPP_EXTERN void CopyCapabilityDataOut( TPMS_CAPABILITY_DATA *capabilityData, void **otherData );
extern CPP_EXTERN void CopyNvPublicOut( TPM2B_NV_PUBLIC *nvPublic, void **otherData );
extern CPP_EXTERN void CopyPcrSelectionIn( void **otherData, TPML_PCR_SELECTION *pcrSelection );
extern CPP_EXTERN void CopyPcrSelectionOut( TPML_PCR_SELECTION *pcrSelection, void **otherData );
extern CPP_EXTERN void CopyDigestIn( void **otherData, TPML_DIGEST *pTpmlDigest );
extern CPP_EXTERN void CopyDigestOut( TPML_DIGEST *pTpmlDigest, void **otherData );
extern CPP_EXTERN void CopySignatureIn( void **otherData, TPMT_SIGNATURE *signature );
extern CPP_EXTERN void CopySignatureOut( TPMT_SIGNATURE *signature, void **otherData );
extern CPP_EXTERN void CopyTpmlAlgOut( TPML_ALG *algList, void **otherData );
extern CPP_EXTERN void CopyTpmlAlgIn( void **otherData, TPML_ALG *algList );
extern CPP_EXTERN void CopySymDefObjectIn( void **otherData, TPMT_SYM_DEF_OBJECT *symDefObject );
extern CPP_EXTERN void CopySigSchemeIn( void **otherData, TPMT_SIG_SCHEME *inScheme );
extern CPP_EXTERN void CopyRSAEncryptIn( void **otherData, TPMT_RSA_DECRYPT *inScheme );
extern CPP_EXTERN void CopyRSADecryptIn( void **otherData, TPMT_RSA_DECRYPT *inScheme );
extern CPP_EXTERN void CopyKDFSchemeIn( void **otherData, TPMT_KDF_SCHEME *kdfScheme );
extern CPP_EXTERN void CopyECCSchemeIn( void **otherData, TPMT_ECC_SCHEME *eccScheme );
extern CPP_EXTERN void CopyAlgorithmicDetailECCIn( void **otherData, TPMS_ALGORITHM_DETAIL_ECC *algDetail );
extern CPP_EXTERN void CopyTpmlCCIn( void **otherData, TPML_CC *ccList );
extern CPP_EXTERN void CopyTpmlCCOut( TPML_CC *ccList, void **otherData );
extern CPP_EXTERN void CopyTpmtHAOut( TPMT_HA *hashStructure, void **otherData );
extern CPP_EXTERN void CopyTpmsContextIn( void **otherData, TPMS_CONTEXT *context );
extern CPP_EXTERN void CopyTpmsContextOut( TPMS_CONTEXT *context, void **otherData );
extern CPP_EXTERN void CopyTimeInfoOut( TPMS_TIME_INFO *timeInfo, void **otherData );
extern CPP_EXTERN void CopyTpmtPublicParmsIn( void **otherData, TPMT_PUBLIC_PARMS *parameters );



extern CPP_EXTERN void TpmErrorHandler();
extern CPP_EXTERN void copyData( UINT8 *to, UINT8 *from, UINT32 length );

extern CPP_EXTERN int SendandReceiveTpmCommand(
    UINT32, UINT32, unsigned int, unsigned char *,
	 unsigned int, UINT32 *, unsigned char *
    );
	
extern CPP_EXTERN int sendTpmCommand(UINT32, UINT32, unsigned int, unsigned char * );
extern CPP_EXTERN int receiveTpmResponse(UINT32, UINT32, unsigned int, UINT32 *, unsigned char *);

#pragma pack ()
#endif
