#ifndef TSS_SYSAPI_H
#define TSS_SYSAPI_H
#pragma pack (1)

#define MAX_SESSIONS 3 
#define CPP_EXTERN   

// Input structure for session data for a single session,
typedef struct {
    TPMI_SH_AUTH_SESSION sessionHandle;
    TPM2B_NONCE *nonce;
    TPMA_SESSION sessionAttributes;
    TPM2B_AUTH *hmac;
} TPM_CMD_SESSION_DATA_IN ;

// Input structure for sessions data.
typedef struct {
    UINT8 numSessions;
    TPM_CMD_SESSION_DATA_IN sessions[MAX_SESSIONS];
} TPM_CMD_SESSIONS_DATA_IN;

// Output structure for session data for a single session.
typedef struct {
    TPM2B_NONCE *nonceTpm;
    TPMA_SESSION sessionAttributes;
    TPM2B_AUTH *hmac;
} TPM_CMD_SESSION_DATA_OUT;

// Output structure for sessions data.
typedef struct {
    UINT8 numSessions;
    TPM_CMD_SESSION_DATA_OUT sessions[MAX_SESSIONS];
} TPM_CMD_SESSIONS_DATA_OUT;



enum POWER_STATE { TPM_AWAKE, TPM_ASLEEP };    


//
// TPM 2.0 Functions
//
// NOTES:
// 1.  The tabHandle and tpmHandle
// 1.  For descriptions of the inputs to and outputs from
//     these functions, please look at Part 3 of the TPM
//     2.0 spec.
// 2.  The return code from these commands is
//     the responseCode as described in the TPM 2.0
//     specs.
// 3.  The sessions data structures that, in Part 3 of the
//     TPM 2.0 spec, are invisible inputs and outputs to
//     most of the TPM 2.0 commands are included in the
//     data structures pointed to by the pSessionsDataIn
//     and pSessionsDataOut pointers below.  For a description
//     of the fields in these data structures, look in Parts
//     1 and 2 of the TPM 2.0 spec.
//

extern CPP_EXTERN UINT32 tspi_sys_TPM2_Startup(
    UINT32 tabHandle,                           /* in */
    UINT32 tpmHandle,                           /* in */
    TPM_SU startupType                          /* in */
    );

extern CPP_EXTERN UINT32 tspi_sys_TPM2_Shutdown(
    UINT32 tabHandle,                           /* in */
    UINT32 tpmHandle,                           /* in */
    TPM_CMD_SESSIONS_DATA_IN *pSessionsDataIn,   /* in */
    TPM_SU shutdownType,                        /* in */

    TPM_CMD_SESSIONS_DATA_OUT *pSessionsDataOut  /* out */
    );

extern CPP_EXTERN UINT32 tspi_sys_TPM2_SelfTest(
    UINT32 tabHandle,                           /* in */
    UINT32 tpmHandle,                           /* in */
    TPM_CMD_SESSIONS_DATA_IN *pSessionsDataIn,   /* in */
    TPMI_YES_NO fullTest,                       /* in */

    TPM_CMD_SESSIONS_DATA_OUT *pSessionsDataOut  /* out */
    );




extern CPP_EXTERN UINT32 tspi_sys_TPM2_StartAuthSession(
    UINT32 tabHandle,                           /* in */
    UINT32 tpmHandle,                           /* in */
    TPMI_DH_OBJECT tpmKey,                      /* in */
    TPMI_DH_ENTITY bind,                        /* in */
    TPM_CMD_SESSIONS_DATA_IN *pSessionsDataIn,   /* in */
    TPM2B_NONCE *pNonceCaller,                  /* in */
    TPM2B_ENCRYPTED_SECRET *pSalt,              /* in */
    TPM_SE sessionType,                         /* in */
    TPMT_SYM_DEF *pSymmetric,                   /* in */
    TPMI_ALG_HASH authHash,                     /* in */

    TPMI_SH_AUTH_SESSION *sessionHandle,        /* out */
    TPM2B_NONCE *pNonceTPM,                     /* out */
    TPM_CMD_SESSIONS_DATA_OUT *pSessionsDataOut  /* out */
    );



extern CPP_EXTERN UINT32 tspi_sys_TPM2_Create (
    UINT32 tabHandle,                           /* in */
    UINT32 tpmHandle,                           /* in */
    TPMI_DH_OBJECT parentHandle,                /* in */
    TPM_CMD_SESSIONS_DATA_IN *pSessionsDataIn,   /* in */
    TPM2B_SENSITIVE_CREATE *pInSensitive,       /* in */
    TPM2B_PUBLIC *pInPublic,                    /* in */
    TPM2B_DATA *pOutsideInfo,                   /* in */
    TPML_PCR_SELECTION *pCreationPCR,           /* in */

    TPM2B_PRIVATE *pOutPrivate,                 /* out */
    TPM2B_PUBLIC *pOutPublic,                   /* out */
    TPM2B_CREATION_DATA *pCreationData,         /* out */
    TPM2B_DIGEST *pCreationHash,                /* out */
    TPMT_TK_CREATION *pCreationTicket,          /* out */
    TPM_CMD_SESSIONS_DATA_OUT *pSessionsDataOut  /* out */
    );

extern CPP_EXTERN UINT32 tspi_sys_TPM2_Load(
    UINT32 tabHandle,                           /* in */
    UINT32 tpmHandle,                           /* in */
    TPMI_DH_OBJECT parentHandle,                /* in */
    TPM_CMD_SESSIONS_DATA_IN *pSessionsDataIn,   /* in */
    TPM2B_PRIVATE *pInPrivate,                  /* in */
    TPM2B_PUBLIC *pInPublic,                    /* in */

    TPM_HANDLE *objectHandle,                   /* out */
    TPM2B_NAME *name,                           /* out */
    TPM_CMD_SESSIONS_DATA_OUT *pSessionsDataOut  /* out */
    );

extern CPP_EXTERN UINT32 tspi_sys_TPM2_LoadExternal(
    UINT32 tabHandle,                           /* in */
    UINT32 tpmHandle,                           /* in */
    TPM_CMD_SESSIONS_DATA_IN *pSessionsDataIn,   /* in */
    TPM2B_SENSITIVE *pInPrivate,                  /* in */
    TPM2B_PUBLIC *pInPublic,                    /* in */
    TPMI_RH_HIERARCHY hierarchy,                /* in */

    TPM_HANDLE *objectHandle,                   /* out */
    TPM2B_NAME *name,                           /* out */
    TPM_CMD_SESSIONS_DATA_OUT *pSessionsDataOut  /* out */
    );

extern CPP_EXTERN UINT32 tspi_sys_TPM2_ReadPublic(
    UINT32 tabHandle,                           /* in */
    UINT32 tpmHandle,                           /* in */
    TPMI_DH_OBJECT objectHandle,                /* in */
    TPM_CMD_SESSIONS_DATA_IN *pSessionsDataIn,   /* in */

    TPM2B_PUBLIC *outPublic,                    /* out */
    TPM2B_NAME *name,                           /* out */
    TPM2B_NAME *qualifiedName,                  /* out */
    TPM_CMD_SESSIONS_DATA_OUT *pSessionsDataOut  /* out */
    );



extern CPP_EXTERN UINT32 tspi_sys_TPM2_Unseal(
    UINT32 tabHandle,                           /* in */
    UINT32 tpmHandle,                           /* in */
    TPMI_DH_OBJECT itemHandle,                  /* in */
    TPM_CMD_SESSIONS_DATA_IN *pSessionsDataIn,   /* in */

    TPM2B_SENSITIVE_DATA *outData,               /* out */
    TPM_CMD_SESSIONS_DATA_OUT *pSessionsDataOut  /* out */
    );

extern CPP_EXTERN UINT32 tspi_sys_TPM2_Hash(
    UINT32 tabHandle,                           /* in */
    UINT32 tpmHandle,                           /* in */
    TPM_CMD_SESSIONS_DATA_IN *pSessionsDataIn,   /* in */
    TPM2B_MAX_BUFFER *data,                     /* in */
    TPMI_ALG_HASH hashAlg,                      /* in */
    TPMI_RH_HIERARCHY hierarchy,                /* in */

    TPM2B_DIGEST *outHash,                      /* out */
    TPMT_TK_HASHCHECK *validation,              /* out */
    TPM_CMD_SESSIONS_DATA_OUT *pSessionsDataOut  /* out */
    );

extern CPP_EXTERN UINT32 tspi_sys_TPM2_HMAC(
    UINT32 tabHandle,                           /* in */
    UINT32 tpmHandle,                           /* in */
    TPMI_DH_OBJECT handle,                      /* in */
    TPM_CMD_SESSIONS_DATA_IN *pSessionsDataIn,   /* in */
    TPM2B_MAX_BUFFER *buffer,                   /* in */
    TPMI_ALG_HASH hashAlg,                      /* in */

    TPM2B_DIGEST *outHMAC,                      /* out */
    TPM_CMD_SESSIONS_DATA_OUT *pSessionsDataOut  /* out */
    );

extern CPP_EXTERN UINT32 tspi_sys_TPM2_GetRandom(
    UINT32 tabHandle,                           /* in */
    UINT32 tpmHandle,                           /* in */
    TPM_CMD_SESSIONS_DATA_IN *pSessionsDataIn,   /* in */
    UINT16 bytesRequested,                      /* in */

    TPM2B_DIGEST *randomBytes,                  /* out */
    TPM_CMD_SESSIONS_DATA_OUT *pSessionsDataOut  /* out */
    );


extern CPP_EXTERN UINT32 tspi_sys_TPM2_HashSequenceStart(
    UINT32 tabHandle,                           /* in */
    UINT32 tpmHandle,                           /* in */
    TPM_CMD_SESSIONS_DATA_IN *pSessionsDataIn,   /* in */
    TPM2B_AUTH *auth,                           /* in */
    TPMI_ALG_HASH hashAlg,                      /* in */

    TPMI_DH_OBJECT *sequenceHandle,             /* out */
    TPM_CMD_SESSIONS_DATA_OUT *pSessionsDataOut  /* out */
    );


extern CPP_EXTERN UINT32 tspi_sys_TPM2_GetSessionAuditDigest(
    UINT32 tabHandle,                           /* in */
    UINT32 tpmHandle,                           /* in */
    TPMI_RH_ENDORSEMENT privacyAdminHandle,     /* in */
    TPMI_DH_OBJECT signHandle,                  /* in */
    TPMI_SH_HMAC sessionHandle,                 /* in */
    TPM_CMD_SESSIONS_DATA_IN *pSessionsDataIn,   /* in */
    TPM2B_DATA *qualifyingData,                 /* in */
    TPMT_SIG_SCHEME *inScheme,                  /* in */

    TPM2B_ATTEST *auditInfo,                    /* out */
    TPMT_SIGNATURE *signature,                  /* out */
    TPM_CMD_SESSIONS_DATA_OUT *pSessionsDataOut  /* out */
    );



extern CPP_EXTERN UINT32 tspi_sys_TPM2_PcrExtend(
    UINT32 tabHandle,                           /* in */
    UINT32 tpmHandle,                           /* in */
    TPMI_DH_PCR pcrHandle,                      /* in */
    TPM_CMD_SESSIONS_DATA_IN *pSessionsDataIn,   /* in */
    TPML_DIGEST_VALUES *digests,                /* in */

    TPM_CMD_SESSIONS_DATA_OUT *pSessionsDataOut  /* out */
    );

extern CPP_EXTERN UINT32 tspi_sys_TPM2_PcrEvent(
    UINT32 tabHandle,                           /* in */
    UINT32 tpmHandle,                           /* in */
    TPMI_DH_PCR pcrHandle,                      /* in */
    TPM_CMD_SESSIONS_DATA_IN *pSessionsDataIn,   /* in */
    TPM2B_EVENT *eventData,                     /* in */

    TPML_DIGEST_VALUES *digests,                /* out */
    TPM_CMD_SESSIONS_DATA_OUT *pSessionsDataOut  /* out */
    );

extern CPP_EXTERN UINT32 tspi_sys_TPM2_PcrRead(
    UINT32 tabHandle,                           /* in */
    UINT32 tpmHandle,                           /* in */
    TPM_CMD_SESSIONS_DATA_IN *pSessionsDataIn,   /* in */
    TPML_PCR_SELECTION *pcrSelectionIn,         /* in */

    UINT32 *pcrUpdateCounter,                   /* out */
    TPML_PCR_SELECTION *pcrSelectionOut,        /* out */
    TPML_DIGEST *pcrValues,                     /* out */
    TPM_CMD_SESSIONS_DATA_OUT *pSessionsDataOut  /* out */
    );

extern CPP_EXTERN UINT32 tspi_sys_TPM2_PcrAllocate(
    UINT32 tabHandle,                           /* in */
    UINT32 tpmHandle,                           /* in */
    TPMI_RH_PLATFORM authHandle,                /* in */
    TPM_CMD_SESSIONS_DATA_IN *pSessionsDataIn,   /* in */
    TPML_PCR_SELECTION *pcrAllocation,          /* in */

    TPMI_YES_NO *allocationSuccess,             /* out */
    UINT32 *maxPcr,                             /* out */
    UINT32 *sizeNeeded,                         /* out */
    UINT32 *sizeAvailable,                      /* out */
    TPM_CMD_SESSIONS_DATA_OUT *pSessionsDataOut  /* out */
    );

extern CPP_EXTERN UINT32 tspi_sys_TPM2_CreatePrimary(
    UINT32 tabHandle,                           /* in */
    UINT32 tpmHandle,                           /* in */
    TPMI_RH_HIERARCHY primaryHandle,            /* in */
    TPM_CMD_SESSIONS_DATA_IN *pSessionsDataIn,   /* in */
    TPM2B_SENSITIVE_CREATE *inSensitive,        /* in */
    TPM2B_PUBLIC *inPublic,                     /* in */
    TPM2B_DATA *outsideInfo,                    /* in */
    TPML_PCR_SELECTION *creationPcr,            /* in */

    TPM_HANDLE *objectHandle,                   /* out */
    TPM2B_PUBLIC *outPublic,                    /* out */
    TPM2B_CREATION_DATA *creationData,          /* out */
    TPM2B_DIGEST *creationHash,                 /* out */
    TPMT_TK_CREATION *creationTicket,           /* out */
    TPM2B_NAME *name,                           /* out */
    TPM_CMD_SESSIONS_DATA_OUT *pSessionsDataOut  /* out */
    );

extern CPP_EXTERN UINT32 tspi_sys_TPM2_HierarchyControl(
    UINT32 tabHandle,                           /* in */
    UINT32 tpmHandle,                           /* in */
    TPMI_RH_HIERARCHY authHandle,               /* in */
    TPM_CMD_SESSIONS_DATA_IN *pSessionsDataIn,   /* in */
    TPMI_RH_HIERARCHY hierarchy,                /* in */
    TPMI_YES_NO state,                          /* in */

    TPM_CMD_SESSIONS_DATA_OUT *pSessionsDataOut  /* out */
    );


extern CPP_EXTERN UINT32 tspi_sys_TPM2_Clear(
    UINT32 tabHandle,                           /* in */
    UINT32 tpmHandle,                           /* in */
    TPMI_RH_CLEAR authHandle,                   /* in */
    TPM_CMD_SESSIONS_DATA_IN *pSessionsDataIn,   /* in */

    TPM_CMD_SESSIONS_DATA_OUT *pSessionsDataOut  /* out */
    );

extern CPP_EXTERN UINT32 tspi_sys_TPM2_ClearControl(
    UINT32 tabHandle,                           /* in */
    UINT32 tpmHandle,                           /* in */
    TPMI_RH_CLEAR auth,                         /* in */
    TPM_CMD_SESSIONS_DATA_IN *pSessionsDataIn,   /* in */
    TPMI_YES_NO disable,                        /* in */

    TPM_CMD_SESSIONS_DATA_OUT *pSessionsDataOut  /* out */
    );

extern CPP_EXTERN UINT32 tspi_sys_TPM2_HierarchyChangeAuth(
    UINT32 tabHandle,                           /* in */
    UINT32 tpmHandle,                           /* in */
    TPMI_RH_HIERARCHY_AUTH authHandle,          /* in */
    TPM_CMD_SESSIONS_DATA_IN *pSessionsDataIn,   /* in */
    TPM2B_AUTH *newAuth,                        /* in */

    TPM_CMD_SESSIONS_DATA_OUT *pSessionsDataOut  /* out */
    );



extern CPP_EXTERN UINT32 tspi_sys_TPM2_FlushContext(
    UINT32 tabHandle,                           /* in */
    UINT32 tpmHandle,                           /* in */
    TPMI_DH_CONTEXT flushHandle                 /* in */
    );


extern CPP_EXTERN UINT32 tspi_sys_TPM2_NV_DefineSpace(
    UINT32 tabHandle,                           /* in */
    UINT32 tpmHandle,                           /* in */
    TPMI_RH_PROVISION authHandle,               /* in */
    TPM_CMD_SESSIONS_DATA_IN *pSessionsDataIn,   /* in */
    TPM2B_AUTH *auth,                           /* in */
    TPM2B_NV_PUBLIC *publicInfo,                /* in */

    TPM_CMD_SESSIONS_DATA_OUT *pSessionsDataOut  /* out */
    );

extern CPP_EXTERN UINT32 tspi_sys_TPM2_NV_UndefineSpace(
    UINT32 tabHandle,                           /* in */
    UINT32 tpmHandle,                           /* in */
    TPMI_RH_PROVISION authHandle,               /* in */
    TPMI_RH_NV_INDEX nvIndex,                   /* in */
    TPM_CMD_SESSIONS_DATA_IN *pSessionsDataIn,   /* in */

    TPM_CMD_SESSIONS_DATA_OUT *pSessionsDataOut  /* out */
    );

extern CPP_EXTERN UINT32 tspi_sys_TPM2_NV_UndefineSpaceSpecial(
    UINT32 tabHandle,                           /* in */
    UINT32 tpmHandle,                           /* in */
    TPMI_RH_NV_INDEX nvIndex,                   /* in */
    TPMI_RH_PLATFORM platform,                  /* in */
    TPM_CMD_SESSIONS_DATA_IN *pSessionsDataIn,   /* in */

    TPM_CMD_SESSIONS_DATA_OUT *pSessionsDataOut  /* out */
    );

extern CPP_EXTERN UINT32 tspi_sys_TPM2_NV_ReadPublic(
    UINT32 tabHandle,                           /* in */
    UINT32 tpmHandle,                           /* in */
    TPMI_RH_NV_INDEX nvIndex,                   /* in */
    TPM_CMD_SESSIONS_DATA_IN *pSessionsDataIn,   /* in */

    TPM2B_NV_PUBLIC *nvPublic,                  /* out */
    TPM2B_NAME *nvName,                         /* out */
    TPM_CMD_SESSIONS_DATA_OUT *pSessionsDataOut  /* out */
    );

extern CPP_EXTERN UINT32 tspi_sys_TPM2_NV_Write(
    UINT32 tabHandle,                           /* in */
    UINT32 tpmHandle,                           /* in */
    TPMI_RH_PROVISION authHandle,               /* in */
    TPMI_RH_NV_INDEX nvIndex,                   /* in */
    TPM_CMD_SESSIONS_DATA_IN *pSessionsDataIn,   /* in */
    TPM2B_MAX_NV_BUFFER *data,                  /* in */
    UINT16 offset,                              /* in */

    TPM_CMD_SESSIONS_DATA_OUT *pSessionsDataOut  /* out */
    );


extern CPP_EXTERN UINT32 tspi_sys_TPM2_NV_WriteLock(
    UINT32 tabHandle,                           /* in */
    UINT32 tpmHandle,                           /* in */
    TPMI_RH_PROVISION authHandle,               /* in */
    TPMI_RH_NV_INDEX nvIndex,                   /* in */
    TPM_CMD_SESSIONS_DATA_IN *pSessionsDataIn,   /* in */

    TPM_CMD_SESSIONS_DATA_OUT *pSessionsDataOut  /* out */
    );


extern CPP_EXTERN UINT32 tspi_sys_TPM2_NV_Read(
    UINT32 tabHandle,                           /* in */
    UINT32 tpmHandle,                           /* in */
    TPMI_RH_PROVISION authHandle,               /* in */
    TPMI_RH_NV_INDEX nvIndex,                   /* in */
    TPM_CMD_SESSIONS_DATA_IN *pSessionsDataIn,   /* in */
    UINT16 size,                                /* in */
    UINT16 offset,                              /* in */

    TPM2B_MAX_NV_BUFFER *data,                  /* out */
    TPM_CMD_SESSIONS_DATA_OUT *pSessionsDataOut  /* out */
    );

extern CPP_EXTERN UINT32 tspi_sys_TPM2_NV_ReadLock(
    UINT32 tabHandle,                           /* in */
    UINT32 tpmHandle,                           /* in */
    TPMI_RH_PROVISION authHandle,               /* in */
    TPMI_RH_NV_INDEX nvIndex,                   /* in */
    TPM_CMD_SESSIONS_DATA_IN *pSessionsDataIn,   /* in */

    TPM_CMD_SESSIONS_DATA_OUT *pSessionsDataOut  /* out */
    );


#pragma pack ()
#endif
